@@ -13,7 +13,6 @@ from .util import *
 blueprint = Blueprint('blueprint', __name__)
 
 dataset_api = DatasetAPI()
-
 auth_api = AuthAPI()
 database_api = DatabaseAPI()
 
@@ -171,38 +170,35 @@ def handle_schema():
     dataset_id = request.args.get('id', '')
     dataset = database_api.get_dataset(email, dataset_id)
     schema = dataset_api.schema(dataset)
+    schema.update(dataset)  # add title, etc from database to schema
     schema['markers'] = database_api.get_feature_sets(email=email, dataset_id=dataset_id)
     return to_json(schema)
 
 
-@blueprint.route('/file', methods=['GET'])
-def handle_file():
-    email = auth_api.auth()['email']
-    dataset_id = request.args.get('id', '')
-    dataset = database_api.get_dataset(email, dataset_id)
-    file = request.args.get('file')
-    url = dataset['url']
-
+def get_file_path(file, dataset_url):
     import os
-    import mimetypes
     # when serving dataset, image must be relative to dataset directory
     if os.environ.get(CIRRO_SERVE) == 'true':
-        _, ext = os.path.splitext(url)
+        _, ext = os.path.splitext(dataset_url)
         if ext != '':
-            url = os.path.dirname(url)
+            url = os.path.dirname(dataset_url)
         if file[0] == '/' or file.find('..') != -1:
             raise ValueError('Incorrect path')
-        file_path = os.path.join(url, file)
+        file_path = os.path.join(dataset_url, file)
     else:
         file_path = file
         pr = urlparse(file_path)
         if (pr.scheme == '' or pr.scheme == 'file') and not os.path.exists(file_path):
-            _, ext = os.path.splitext(url)
+            _, ext = os.path.splitext(dataset_url)
             if ext != '':
-                url = os.path.dirname(url)
-            file_path = os.path.join(url, file)
+                url = os.path.dirname(dataset_url)
+            file_path = os.path.join(dataset_url, file)
+    return file_path
 
-    mimetype = mimetypes.guess_type(file_path)
+
+def send_file(file_path):
+    import mimetypes
+    mimetype, encoding = mimetypes.guess_type(file_path)
     # with dataset_api.fs_adapter.get_fs(file_path).open(file_path) as f:
     #     bytes = f.read()
     # return Response(bytes, mimetype=mimetype[0])
@@ -217,8 +213,18 @@ def handle_file():
                 break
             yield chunk
 
-    response = Response(stream_with_context(generate()), mimetype=mimetype[0])
-    return response
+    r = Response(stream_with_context(generate()), mimetype=mimetype)
+    r.headers["Content-Encoding"] = encoding
+    return r
+
+
+@blueprint.route('/file', methods=['GET'])
+def handle_file():
+    email = auth_api.auth()['email']
+    dataset_id = request.args.get('id', '')
+    dataset = database_api.get_dataset(email, dataset_id)
+    file_path = get_file_path(request.args.get('file'), dataset['url'])
+    return send_file(file_path)
 
 
 @blueprint.route('/user', methods=['GET'])
@@ -245,6 +251,7 @@ def handle_data():
     return to_json(
         data_processing.handle_data(dataset_api=dataset_api, dataset=dataset,
             embedding_list=json_request.get('embedding'),
+            values=json_request.get('values'),
             stats=json_request.get('stats'),
             grouped_stats=json_request.get('groupedStats'),
             selection=json_request.get('selection')))
@@ -286,15 +293,19 @@ def handle_dataset():
         dataset_id = content.get('id', '')
         if request.method == 'PUT' and dataset_id == '':
             return 'Please supply an id', 400
-        readers = set(content.get('readers', []))
+        readers = content.get('readers', [])
+        if readers is None:
+            readers = []
+        readers = set(readers)
         dataset_name = content.get('name', '')
         description = content.get('description', '')
+        title = content.get('title', '')
         url = content.get('url', '')  # e.g. gs://foo/a/b/
         if dataset_name == '' or url == '':
             return 'Must supply dataset name and URL', 400
         dataset_id = database_api.upsert_dataset(email=email,
             dataset_id=dataset_id if request.method == 'PUT' else None,
-            dataset_name=dataset_name, url=url, readers=readers, description=description)
+            dataset_name=dataset_name, url=url, readers=readers, description=description, title=title)
         return to_json({'id': dataset_id})
     elif request.method == 'DELETE':
         content = request.get_json(force=True, cache=False)
@@ -320,3 +331,40 @@ def handle_dataset():
     # head_request.close()
     # if head_request.status_code != 200:
     #     return 'Not authorized to read {}'.format(url), 403
+
+
+# @blueprint.route('/submit_job', methods=['POST'])
+# def handle_submit_job():
+#     content = request.get_json(force=True, cache=False)
+#     email, dataset = get_email_and_dataset(content)
+#     params = content.get('params')
+#     job_type = content.get('type')
+#     job_name = content.get('name')
+#     return dict(id=submit_job(database_api=database_api, dataset_api=dataset_api, email=email, dataset=dataset,
+#         job_name=job_name, job_type=job_type, params=params))
+
+
+@blueprint.route('/job', methods=['GET'])
+def handle_job():
+    email = auth_api.auth()['email']
+    job_id = request.args.get('id', '')
+    job_status = request.args.get('status', '0')
+    return_result = job_status == '0'
+    if job_id.startswith('cirro-'):  # precomputed result
+        import os
+        dataset_id = request.args.get('ds_id', '')
+        email = auth_api.auth()['email']
+        dataset = database_api.get_dataset(email, dataset_id)
+        file_path = get_file_path(os.path.join('uns', job_id + '.json.gz'), dataset['url'])
+        return send_file(file_path)
+    result = database_api.get_job(email=email, job_id=job_id, return_result=return_result)
+    if return_result:
+        result = Response(result, mimetype='application/json')
+        result.headers["Content-Encoding"] = 'gzip'
+    return result
+
+
+@blueprint.route('/jobs', methods=['GET'])
+def handle_jobs():
+    email = auth_api.auth()['email']
+    return database_api.get_jobs(email=email)
