@@ -126,18 +126,16 @@ def get_var_name_type(key):
         return name, key_type
 
 
-def split_measures(measures):
-    obs_measures = []
-    var_measures = []
+def get_type_to_measures(measures):
+    type2measures = dict(X=[], obs=[])
     for measure in measures:
         name, key_type = get_var_name_type(measure)
-        if key_type == 'X':
-            var_measures.append(name)
-        elif key_type == 'obs':
-            obs_measures.append(name)
-        else:
-            raise ValueError('Unknown key type: ' + key_type)
-    return obs_measures, var_measures
+        type_measures = type2measures.get(key_type)
+        if type_measures is None:
+            type_measures = []
+            type2measures[key_type] = type_measures
+        type_measures.append(name)
+    return type2measures
 
 
 def check_bin_input(nbins):
@@ -159,12 +157,12 @@ def handle_export_dataset_filters(dataset_api, dataset, data_filters):
         filter_names.append(data_filter_obj['name'])
         reformatted_filters.append(filter_value)
 
-    df_info = get_data(dataset_api, dataset, measures=['obs/index'], data_filters=reformatted_filters)
-    result_df = pd.DataFrame(index=df_info['df']['index'])
+    df = get_df(dataset_api, dataset, measures=['obs/index'], data_filters=reformatted_filters)
+    result_df = pd.DataFrame(index=df['index'])
     for i in range(len(reformatted_filters)):
         data_filter = reformatted_filters[i]
         filter_name = filter_names[i]
-        df_filtered = apply_filter(df_info['df'], data_filter)
+        df_filtered = apply_filter(df, data_filter)
         df = pd.DataFrame(index=df_filtered['index'])
         df[filter_name] = True
         result_df = result_df.join(df, rsuffix='r')
@@ -172,29 +170,34 @@ def handle_export_dataset_filters(dataset_api, dataset, data_filters):
     return result_df.to_csv()
 
 
-# embedding - list of basis, measures, dimensions
-def handle_data(dataset_api, dataset, embedding_list=None, grouped_stats=None, stats=None, selection=None):
+# embedding - list of basis and coords (whether to return coordinates). For binned embeddings, also measures and dimensions.
+def handle_data(dataset_api, dataset, embedding_list=None, values=None, grouped_stats=None, stats=None, selection=None):
     dimensions = set()
     measures = set()
     basis_list = []
     basis_keys = set()
     if embedding_list is not None:
         for embedding in embedding_list:
-            dimensions.update(embedding.get('dimensions', []))
-            embedding['count'] = '__count' in embedding.get('measures', [])
-            measures.update(embedding.get('measures', []))
+            precomputed = embedding.get('precomputed', False)
             nbins = check_bin_input(embedding.get('nbins', None))
             agg_function = embedding.get('agg', 'max')
             ndim = embedding.get('ndim', '2')
-            precomputed = embedding.get('precomputed', False)
+            coords = embedding.get('coords', True)
             basis = get_basis(embedding.get('basis'), nbins=nbins, agg=agg_function, dimensions=ndim,
-                precomputed=precomputed)
+                precomputed=precomputed, coords=coords)
+            if nbins is not None:
+                dimensions.update(embedding.get('dimensions', []))
+                measures.update(embedding.get('measures', []))
 
             if basis['full_name'] not in basis_keys:
                 basis_list.append(basis)
                 basis_keys.add(basis['full_name'])
 
             embedding['basis'] = basis
+    if values is not None:
+        dimensions.update(values.get('dimensions', []))
+        measures.update(values.get('measures', []))
+
     if selection is not None:
         data_filter = selection.get('filter')
         var_keys_filter, obs_keys_filter, selected_points_filter_basis_list = data_filter_keys(data_filter)
@@ -203,9 +206,9 @@ def handle_data(dataset_api, dataset, embedding_list=None, grouped_stats=None, s
         dimensions.update(obs_keys_filter)
         dimensions.update(selection.get('dimensions', []))
         measures.update(selection.get('measures', []))
-        embeddings = selection.get('embeddings', [])
+        selection_embeddings = selection.get('embeddings', [])
         embeddings_mapped = []
-        for embedding in embeddings:
+        for embedding in selection_embeddings:
             basis = get_basis(embedding.get('basis'), nbins=embedding.get('nbins'), agg=embedding.get('agg', 'max'),
                 dimensions=embedding.get('ndim', '2'),
                 precomputed=embedding.get('precomputed', False))
@@ -228,73 +231,90 @@ def handle_data(dataset_api, dataset, embedding_list=None, grouped_stats=None, s
         dimensions.update(stats.get('dimensions', []))
         measures.update(stats.get('measures', []))
 
-    has_count = '__count' in measures
-    if has_count:
-        measures.remove('__count')
-    obs_measures, var_measures = split_measures(measures)
-    df = dataset_api.read(dataset=dataset, obs_keys=list(dimensions) + obs_measures,
-        var_keys=var_measures, basis=basis_list)
-    if has_count:
-        df['__count'] = 1
+    keys = get_type_to_measures(measures)
+    keys['obs'] += list(dimensions)
+    keys['basis'] = basis_list
+    df = dataset_api.read_dataset(dataset=dataset, keys=keys)
+    for basis_obj in basis_list:
+        if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
+            EmbeddingAggregator.convert_coords_to_bin(df=df,
+                nbins=basis_obj['nbins'],
+                bin_name=basis_obj['full_name'],
+                coordinate_columns=basis_obj['coordinate_columns'])
     results = {}
+    if values is not None:
+        dimensions = values.get('dimensions', [])
+        measures = values.get('measures', [])
+        type2measures = get_type_to_measures(measures)
+        measures = []
+        for v in type2measures.values():
+            measures += v
+
+        result = EmbeddingAggregator(
+            measures=measures,
+            dimensions=dimensions,
+            nbins=None,
+            basis=None,
+            coords=False,
+            agg_function=None,
+            quick=True).execute(df)
+        results['values'] = result['values']
+
     if embedding_list is not None:
-        results['embedding'] = []
+        results['embeddings'] = []
         for embedding in embedding_list:
             dimensions = embedding.get('dimensions', [])
             measures = embedding.get('measures', [])
-            obs_measures, var_measures = split_measures(measures)
+            type2measures = get_type_to_measures(measures)
             basis = embedding['basis']
             if basis['precomputed']:
-                result = precomputed_embedding(dataset_api, dataset, basis, obs_measures, var_measures, dimensions)
+                result = precomputed_embedding(dataset_api, dataset, basis, type2measures['obs'],
+                    type2measures['X'], dimensions)
             else:
-                result = EmbeddingAggregator(obs_measures=obs_measures,
-                    var_measures=var_measures,
+                result = EmbeddingAggregator(
+                    measures=type2measures['obs'] + type2measures['X'],
                     dimensions=dimensions,
-                    count=embedding['count'],
                     nbins=basis['nbins'],
                     basis=basis,
+                    coords=basis['coords'],
                     agg_function=basis['agg'],
                     quick=True).execute(df)
-            results['embedding'].append(result)
+            results['embeddings'].append(result)
     if grouped_stats is not None:
         if dataset_api.has_precomputed_stats(dataset):
-            results['dotplot'] = precomputed_grouped_stats(dataset_api, dataset, grouped_stats.get('measures', []),
+            results['distribution'] = precomputed_grouped_stats(dataset_api, dataset, grouped_stats.get('measures', []),
                 grouped_stats.get('dimensions', []))
         else:
-            results['dotplot'] = DotPlotAggregator(var_measures=grouped_stats.get('measures', []),
+            results['distribution'] = DotPlotAggregator(var_measures=grouped_stats.get('measures', []),
                 dimensions=grouped_stats.get('dimensions', [])).execute(df)
     if stats is not None:
         dimensions = stats.get('dimensions', [])
         measures = stats.get('measures', [])
-        obs_measures, var_measures = split_measures(measures)
+        type2measures = get_type_to_measures(measures)
         if dataset_api.has_precomputed_stats(dataset):
-            result['summary'] = precomputed_summary(dataset_api, dataset, obs_measures, var_measures, dimensions)
+            result['summary'] = precomputed_summary(dataset_api, dataset, type2measures['obs'],
+                type2measures['X'], dimensions)
         else:
-            results['summary'] = FeatureAggregator(obs_measures, var_measures, dimensions).execute(df)
+            results['summary'] = FeatureAggregator(type2measures['obs'], type2measures['X'],
+                dimensions).execute(df)
     if selection is not None:
         results['selection'] = {}
         dimensions = selection.get('dimensions', [])
         measures = selection.get('measures', [])
-        obs_measures, var_measures = split_measures(measures)
-        basis_list = selection.get('basis', [])
-        embeddings = selection.get('embeddings', [])
-        for basis_obj in basis_list + embeddings:
-            if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
-                EmbeddingAggregator.convert_coords_to_bin(df=df,
-                    nbins=basis_obj['nbins'],
-                    bin_name=basis_obj['full_name'],
-                    coordinate_columns=basis_obj['coordinate_columns'])
-
+        type2measures = get_type_to_measures(measures)
+        # basis_list = selection.get('basis', [])
+        selection_embeddings = selection.get('embeddings', [])
         df = apply_filter(df, data_filter)
-        if len(embeddings) > 0:
+        if len(selection_embeddings) > 0:
             results['selection']['coordinates'] = {}
-            for embedding in embeddings:
+            for embedding in selection_embeddings:
                 results['selection']['coordinates'][embedding['full_name']] = UniqueAggregator(
                     embedding['full_name'] if embedding['nbins'] is not None else 'index').execute(df)
-
-        results['selection']['summary'] = FeatureAggregator(obs_measures, var_measures, dimensions).execute(df)
+        var_measures = type2measures['X']
+        results['selection']['summary'] = FeatureAggregator(type2measures['obs'], type2measures['X'],
+            dimensions).execute(df)
         if len(dimensions) > 0 and len(var_measures) > 0:
-            results['selection']['dotplot'] = DotPlotAggregator(var_measures=var_measures,
+            results['selection']['distribution'] = DotPlotAggregator(var_measures=var_measures,
                 dimensions=[dimensions]).execute(df)
         results['selection']['count'] = df.shape[0]
 
@@ -302,12 +322,12 @@ def handle_data(dataset_api, dataset, embedding_list=None, grouped_stats=None, s
 
 
 def handle_selection_ids(dataset_api, dataset, data_filter):
-    selection_info = get_selected_data(dataset_api, dataset, measures=['obs/index'], data_filter=data_filter)
-    return {'ids': IdsAggregator().execute(selection_info['df'])}
+    df = get_selected_data(dataset_api, dataset, measures=['obs/index'], data_filter=data_filter)
+    return {'ids': IdsAggregator().execute(df)}
 
 
-def get_data(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filters=[]):
-    obs_measures, var_measures = split_measures(measures)
+def get_df(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filters=[]):
+    type2measures = get_type_to_measures(measures)
     var_keys_filter = []
     obs_keys_filter = []
     selected_points_filter_basis_list = []
@@ -316,8 +336,8 @@ def get_data(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], da
         selected_points_filter_basis_list += selected_points_filter_basis_list
         var_keys_filter += _var_keys_filter
         obs_keys_filter += _obs_keys_filter
-    obs_keys = list(set(obs_measures + obs_keys_filter + dimensions))
-    var_keys = list(set(var_measures + var_keys_filter))
+    obs_keys = list(set(type2measures['obs'] + obs_keys_filter + dimensions))
+    var_keys = list(set(type2measures['X'] + var_keys_filter))
     basis_objs = []
     basis_keys = set()
     for embedding in embeddings:
@@ -330,23 +350,21 @@ def get_data(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], da
         if basis_obj['full_name'] not in basis_keys:
             basis_objs.append(basis_obj)
             basis_keys.add(basis_obj['full_name'])
-    df = dataset_api.read(dataset=dataset, obs_keys=obs_keys, var_keys=var_keys, basis=basis_objs)
+    df = dataset_api.read_dataset(dataset=dataset, keys=dict(obs=obs_keys, X=var_keys, basis=basis_objs))
     for basis_obj in basis_objs:
         if not basis_obj['precomputed'] and basis_obj['nbins'] is not None:
             EmbeddingAggregator.convert_coords_to_bin(df=df,
                 nbins=basis_obj['nbins'],
                 bin_name=basis_obj['full_name'],
                 coordinate_columns=basis_obj['coordinate_columns'])
-    return {'df': df, 'basis': basis_objs, 'obs_measures': obs_measures, 'var_measures': var_measures,
-            'dimensions': dimensions}
+    return df
 
 
 def get_selected_data(dataset_api, dataset, embeddings=[], measures=[], dimensions=[], data_filter=None):
-    df_info = get_data(dataset_api, dataset, embeddings, measures, dimensions,
+    df = get_df(dataset_api, dataset, embeddings, measures, dimensions,
         [data_filter] if data_filter is not None else [])
-    df = apply_filter(df_info['df'], data_filter)
-    df_info['df'] = df
-    return df_info
+    df = apply_filter(df, data_filter)
+    return df
 
 
 def data_filter_keys(data_filter):
