@@ -66,7 +66,9 @@ class MongoDb:
         return {
                 'id': str(doc['_id']),
                 'name': doc['name'],
-                'readers': doc['readers'],
+                'readers': doc.get('readers'),
+                'description': doc.get('description'),
+                'title': doc.get('title'),
                 'url': doc['url'],
                 'owner': 'owners' in doc and email in doc['owners']}
 
@@ -79,7 +81,7 @@ class MongoDb:
         else:
             query = dict(readers={'$in': [email, domain]})
         for doc in collection.find(query):
-            results.append({'id': str(doc['_id']), 'name': doc['name'], 'description': doc.get('description'),
+            results.append({'id': str(doc['_id']), 'name': doc['name'], 'title': doc.get('title'),
                             'owner': 'owners' in doc and email in doc['owners'], 'url': doc['url']})
         return results
 
@@ -134,7 +136,7 @@ class MongoDb:
         self.db.filters.delete_many(dict(dataset_id=dataset_id))
         self.db.categories.delete_many(dict(dataset_id=dataset_id))
 
-    def upsert_dataset(self, email, dataset_id, dataset_name, url, readers, description):
+    def upsert_dataset(self, email, dataset_id, dataset_name, url, readers, description, title):
         collection = self.db.datasets
         readers = set(readers)
         if email in readers:
@@ -142,6 +144,7 @@ class MongoDb:
         readers.add(email)
         update_dict = {'name': dataset_name,
                        'description': description,
+                       'title': title,
                        'readers': list(readers),
                        'url': url}
         if dataset_id is None:  # new dataset
@@ -190,3 +193,33 @@ class MongoDb:
         else:
             collection.update_one(dict(_id=ObjectId(set_id)), {'$set': entity_update})
             return set_id
+
+    def create_job(self, email, dataset_id, job_name, job_type, params):
+        self.get_dataset(email, dataset_id)
+        collection = self.db.jobs
+        return str(collection.insert_one(
+            dict(dataset_id=dataset_id, name=job_name, type=job_type, params=params)).inserted_id)
+
+    def get_job(self, email, job_id, return_result):
+        collection = self.db.jobs
+        doc = collection.find_one(dict(_id=ObjectId(job_id)), {"result": 0} if not return_result else None)
+        self.get_dataset(email, doc['dataset_id'])
+        if return_result:
+            return doc['result']
+        else:
+            return dict(id=str(doc['_id']), status=doc['status'])
+
+    def get_jobs(self, email, dataset_id):
+        self.get_dataset(email, dataset_id)
+        collection = self.db.jobs
+        results = []
+        for doc in collection.find(dict(dataset_id=dataset_id), dict(name=1, status=1)):
+            results.append(
+                dict(id=str(doc['_id']), name=doc['name'], status=doc['status'], type=doc['type']))
+        return results
+
+    def update_job(self, email, job_id, status, result):
+        collection = self.db.jobs
+        doc = collection.find_one(dict(_id=ObjectId(job_id)))
+        self.get_dataset(email, doc['dataset_id'])
+        collection.update_one(dict(_id=ObjectId(job_id)), {'$set': dict(status=status, result=result)})
