@@ -7,10 +7,7 @@ import os
 import numpy as np
 import pandas as pd
 import pandas._libs.json as ujson
-from natsort import natsorted
-from pandas import CategoricalDtype
-
-import cirrocumulus.data_processing as data_processing
+import scipy.sparse
 from cirrocumulus.anndata_dataset import AnndataDataset
 from cirrocumulus.dataset_api import DatasetAPI
 from cirrocumulus.io_util import get_markers, filter_markers, add_spatial, SPATIAL_HELP, unique_id
@@ -18,19 +15,43 @@ from cirrocumulus.simple_data import SimpleData
 
 logger = logging.getLogger("cirro")
 
+cluster_fields = ['seurat_clusters', 'leiden', 'louvain']
+categorical_fields_convert = ['seurat_clusters']
 
-def read_adata(path, backed=False, spatial_directory=None):
+
+def read_adata(path, backed=False, spatial_directory=None, use_raw=False):
     import anndata
     adata = anndata.read_loom(path) if path.lower().endswith('.loom') else anndata.read(path,
         backed=backed)
+    if use_raw and adata.raw is not None and adata.shape[0] == adata.raw.shape[0]:
+        logger.info('Using adata.raw')
+        adata = anndata.AnnData(X=adata.raw.X, var=adata.raw.var, obs=adata.obs, obsm=adata.obsm, uns=adata.uns)
+
     if spatial_directory is not None:
         if not add_spatial(adata, spatial_directory):
             print('No spatial data found in {}'.format(spatial_directory))
     if not backed:
-        sums = adata.X.sum(axis=0)
-        if isinstance(sums, np.matrix):
-            sums = sums.A1
-        adata = adata[:, sums > 0]
+        if scipy.sparse.issparse(adata.X) and scipy.sparse.isspmatrix_csr(adata.X):
+            adata.X = adata.X.tocsc()
+
+    def fix_column_names(df):
+        rename = {}
+        for c in df.columns:
+            if c.find(' ') != -1:
+                rename[c] = c.replace(' ', '_')
+        return df.rename(rename, axis=1) if len(rename) > 0 else df
+
+    adata.obs = fix_column_names(adata.obs)
+    adata.var = fix_column_names(adata.var)
+    for field in categorical_fields_convert:
+        if field in adata.obs and not pd.api.types.is_categorical_dtype(adata.obs[field]):
+            logger.info('Converting {} to categorical'.format(field))
+            adata.obs[field] = adata.obs[field].astype('category')
+    for key in adata.obsm:
+        if key.find(' ') != -1:
+            new_key = key.replace(' ', '_')
+            adata.obsm[new_key] = adata.obsm[key]
+            del adata.obsm[key]
     return adata
 
 
@@ -58,16 +79,6 @@ def make_unique(index, join='-1'):
     return index
 
 
-def make_ordered(df, columns):
-    if columns is None:
-        columns = df.columns
-    for i in range(len(columns)):
-        name = columns[i]
-        c = df[name]
-        if pd.api.types.is_categorical_dtype(c) and not c.dtype.ordered:
-            df[name] = df[name].astype(CategoricalDtype(natsorted(df[name].dtype.categories), ordered=True))
-
-
 def write_obs_stats(directory, results):
     os.makedirs(os.path.join(directory, 'stats', 'obs'), exist_ok=True)
     for column in results:
@@ -168,26 +179,25 @@ def write_basis_obs(basis, coords_group, obs_group, result):
 
 class PrepareData:
 
-    def __init__(self, adata, output, X_range=None, dimensions=None, groups=[], markers=[], output_format='parquet'):
+    def __init__(self, adata, output, dimensions=None, groups=[], group_nfeatures=10, markers=[],
+                 output_format='parquet'):
         self.adata = adata
         self.groups = groups
+        self.group_nfeatures = group_nfeatures
         self.markers = markers
         self.stats = False
         self.nbins = None
         self.bin_agg_function = None
         self.output_format = output_format
+
         # if basis_list is None or len(basis_list) == 0:
         #     basis_list = list(self.adata.obsm_keys())
         self.basis_list_to_precompute = None
 
-        # if 'seurat_clusters' in self.adata.obs:
-        #     self.adata.obs['seurat_clusters'] = self.adata.obs['seurat_clusters'].astype('category')
-
         index = make_unique(self.adata.var.index.append(pd.Index(self.adata.obs.columns)))
         self.adata.var.index = index[0:len(self.adata.var.index)]
         self.adata.obs.columns = index[len(self.adata.var.index):]
 
-        self.X_range = (0, self.adata.shape[1]) if X_range is None else X_range
         self.base_output_dir = output
 
         self.dataset_api = DatasetAPI()
@@ -220,28 +230,51 @@ class PrepareData:
                 self.measures.append('obs/' + name)
             else:
                 self.others.append(name)
-        make_ordered(self.adata.obs, self.dimensions)
 
     def get_path(self, path):
         return os.path.join(self.base_output_dir, path)
 
     def execute(self):
-
         basis_list = self.basis_list_to_precompute
         nbins = self.nbins
+        group_nfeatures = self.group_nfeatures
         bin_agg_function = self.bin_agg_function
-        X_range = self.X_range
+        output_format = self.output_format
         if not os.path.exists(self.base_output_dir):
             os.makedirs(self.base_output_dir, exist_ok=True)
-        markers = self.adata.uns.get('markers', [])
-        self.adata.uns['markers'] = markers
+
+        schema = self.get_schema()
+        results = schema.get('results', [])
+        if len(results) > 0:
+            uns_dir = os.path.join(self.base_output_dir, 'uns')
+            os.makedirs(uns_dir, exist_ok=True)
+            for i in range(len(results)):  # keep id, name, type in schema, store rest in file
+                result = results[i]
+                result_id = result.pop('id')
+                results[i] = dict(id=result_id, name=result.pop('name'), type=result.pop('type'),
+                    content_type='application/json', content_encoding='gzip')
+                with gzip.open(os.path.join(uns_dir, result_id + '.json.gz'), 'wt') as f:
+                    f.write(to_json(result))
+
+        markers = schema.get('markers', [])
+
+        if len(markers) == 0 and self.groups is None:
+            groups = []
+            for field in self.adata.obs.columns:
+                field_lc = field.lower()
+                for cluster_field in cluster_fields:
+                    if field_lc.find(cluster_field) != -1:
+                        groups.append(field)
+                        break
+            self.groups = groups
         if self.groups is not None:
-            n_genes = 10
             for field in self.groups:
+                if not pd.api.types.is_categorical_dtype(self.adata.obs[field]):
+                    self.adata.obs[field] = self.adata.obs[field].astype('category')
                 if len(self.adata.obs[field].cat.categories) > 1:
-                    logger.info('Computing markers for {}'.format(field))
-                    markers += SimpleData.find_markers(self.adata, field, n_genes)
-            self.adata.uns['markers'] = markers
+                    markers += SimpleData.find_markers(self.adata, field, group_nfeatures)
+            schema['markers'] = markers
+
         images = self.adata.uns.get('images')
         if images is not None:
             image_dir = os.path.join(self.base_output_dir, 'images')
@@ -253,16 +286,17 @@ class PrepareData:
                 shutil.copy(path, os.path.join(image_dir, os.path.basename(path)))
                 image['image'] = 'images/' + os.path.basename(path)
 
-        if X_range[0] == 0:
-            schema = self.get_schema()
-            if self.output_format == 'parquet':
-                from cirrocumulus.parquet_io import save_adata_pq
-                save_adata_pq(self.adata, schema, self.base_output_dir)
-            elif self.output_format == 'json':
-                from cirrocumulus.json_io import save_adata_json
-                save_adata_json(self.adata, schema, self.base_output_dir)
-            else:
-                raise ValueError("Unknown format")
+        if output_format == 'parquet':
+            from cirrocumulus.parquet_io import save_adata_pq
+            save_adata_pq(self.adata, schema, self.base_output_dir)
+        elif output_format == 'json':
+            from cirrocumulus.json_io import save_adata_json
+            save_adata_json(self.adata, schema, self.base_output_dir)
+        elif output_format == 'jsonl':
+            from cirrocumulus.jsonl_io import save_adata_jsonl
+            save_adata_jsonl(self.adata, schema, self.base_output_dir)
+        else:
+            raise ValueError("Unknown format")
 
         # if self.stats:
         #     self.summary_stats()
@@ -273,40 +307,40 @@ class PrepareData:
                 self.grid_embedding(basis_name, bin_agg_function, nbins)
 
 
-    def summary_stats(self):
-        dimensions = self.dimensions
-        measures = self.measures
-        dataset_api = self.dataset_api
-        input_dataset = self.input_dataset
-        X_range = self.X_range
-        adata = self.adata
-        base_dir = self.base_output_dir
-
-        if X_range[0] == 0:
-            process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
-                stats=dict(measures=measures, dimensions=dimensions))
-            write_obs_stats(base_dir, process_results['summary'])
-
-        logger.info('Summary stats X {}-{}'.format(X_range[0], X_range[1]))
-        process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
-            stats=dict(measures=adata.var_names[X_range[0]:X_range[1]], dimensions=[]))
-        write_X_stats(base_dir, process_results['summary'])
-        write_X_bins(base_dir)
-
-    # stats, grouped by categories in adata.obs for dot plot
-    def grouped_stats(self):
-        dimensions = self.dimensions
-        dataset_api = self.dataset_api
-        input_dataset = self.input_dataset
-        adata = self.adata
-        X_range = self.X_range
-        base_dir = self.base_output_dir
-        logger.info('Grouped stats X {}-{}'.format(X_range[0], X_range[1]))
-        process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
-            grouped_stats=dict(measures=adata.var_names[X_range[0]:X_range[1]], dimensions=dimensions))
-        dotplot_results = process_results['dotplot']
-
-        write_grouped_stats(base_dir, dotplot_results)
+    # def summary_stats(self):
+    #     dimensions = self.dimensions
+    #     measures = self.measures
+    #     dataset_api = self.dataset_api
+    #     input_dataset = self.input_dataset
+    #     X_range = self.X_range
+    #     adata = self.adata
+    #     base_dir = self.base_output_dir
+    #
+    #     if X_range[0] == 0:
+    #         process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
+    #             stats=dict(measures=measures, dimensions=dimensions))
+    #         write_obs_stats(base_dir, process_results['summary'])
+    #
+    #     logger.info('Summary stats X {}-{}'.format(X_range[0], X_range[1]))
+    #     process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
+    #         stats=dict(measures=adata.var_names[X_range[0]:X_range[1]], dimensions=[]))
+    #     write_X_stats(base_dir, process_results['summary'])
+    #     write_X_bins(base_dir)
+    #
+    # # stats, grouped by categories in adata.obs for dot plot
+    # def grouped_stats(self):
+    #     dimensions = self.dimensions
+    #     dataset_api = self.dataset_api
+    #     input_dataset = self.input_dataset
+    #     adata = self.adata
+    #     X_range = self.X_range
+    #     base_dir = self.base_output_dir
+    #     logger.info('Grouped stats X {}-{}'.format(X_range[0], X_range[1]))
+    #     process_results = data_processing.handle_data(dataset_api=dataset_api, dataset=input_dataset,
+    #         grouped_stats=dict(measures=adata.var_names[X_range[0]:X_range[1]], dimensions=dimensions))
+    #     dotplot_results = process_results['dotplot']
+    #
+    #     write_grouped_stats(base_dir, dotplot_results)
 
 
     # def grid_embedding(self, basis_name, summary, nbins):
@@ -353,10 +387,13 @@ class PrepareData:
         nbins = self.nbins
         bin_agg_function = self.bin_agg_function
         result = SimpleData.schema(self.adata)
-        markers = self.adata.uns.get('markers', [])
-        if self.markers is not None:
+        markers = result.get('markers', [])
+
+        if self.markers is not None:  # add from file
             markers += get_markers(self.markers)
+
         markers = filter_markers(self.adata, markers)
+
         for marker in markers:
             if marker.get('id') is None:
                 marker['id'] = unique_id()
@@ -384,9 +421,12 @@ def main(argsv):
     # parser.add_argument('--basis', help='List of embeddings to precompute', action='append')
     #  parser.add_argument('--groups', help='List of groups to precompute summary statistics', action='append')
     parser.add_argument('--markers',
-        help='Path to JSON file that maps name to features. For example {"a":["gene1", "gene2"], "b":["gene3"]',
+        help='Path to JSON file of precomputed markers that maps name to features. For example {"a":["gene1", "gene2"], "b":["gene3"]',
         action='append')
-    parser.add_argument('--groups', help='List of groups to compute markers for (e.g. louvain)', action='append')
+    parser.add_argument('--groups',
+        help='List of groups to compute markers for (e.g. louvain). Note that markers created with scanpy or cumulus are automatically included.',
+        action='append')
+    parser.add_argument('--group_nfeatures', help='Number of marker genes/features to include', type=int, default=10)
     parser.add_argument('--spatial', help=SPATIAL_HELP)
     # parser.add_argument('--output_format', help='Output file format', choices=['json', 'parquet'], default='parquet')
     # parser.add_argument('--nbins', help='Number of bins. Set to 0 to disable binning', default=500, type=int)
@@ -405,19 +445,22 @@ def main(argsv):
     if out is None:
         out = os.path.basename(input_dataset)
         out = out[0:out.rindex('.')]
-    loom_file = None
+    tmp_file = None
+    use_raw = False
     if input_dataset.lower().endswith('.rds'):
         import subprocess
         import tempfile
         import pkg_resources
 
-        _, loom_file = tempfile.mkstemp(suffix='.loom')
-        os.remove(loom_file)
+        _, h5_file = tempfile.mkstemp(suffix='.h5ad')
+        os.remove(h5_file)
         subprocess.check_call(
-            ['Rscript', pkg_resources.resource_filename("cirrocumulus", 'seurat2loom.R'), input_dataset, loom_file])
-        input_dataset = loom_file
-    summary = 'max'
-    nbins = -1
+            ['Rscript', pkg_resources.resource_filename("cirrocumulus", 'seurat2h5ad.R'), input_dataset, h5_file])
+        input_dataset = h5_file
+        tmp_file = h5_file
+        use_raw = True
+    # summary = 'max'
+    # nbins = -1
     # summary = args.summary
     # if args.basis is not None:
     #     for b in args.basis:
@@ -429,12 +472,13 @@ def main(argsv):
     #     input_X_range[0] = int(input_X_range[0])
     #     input_X_range[1] = int(input_X_range[1])
 
-    adata = read_adata(input_dataset, backed=args.backed, spatial_directory=args.spatial)
+    adata = read_adata(input_dataset, backed=args.backed, spatial_directory=args.spatial, use_raw=use_raw)
     prepare_data = PrepareData(adata=adata, output=out, dimensions=args.groups, groups=args.groups,
+        group_nfeatures=args.group_nfeatures,
         markers=args.markers, output_format=output_format)
     prepare_data.execute()
-    if loom_file is not None:
-        os.remove(loom_file)
+    if tmp_file is not None:
+        os.remove(tmp_file)
 
 
 if __name__ == '__main__':
