@@ -22,11 +22,12 @@ import {
 } from './actions';
 import AppHeader from './AppHeader';
 import DeleteDatasetDialog from './DeleteDatasetDialog';
-import DotPlots from './DotPlots';
+import DistributionPlots from './DistributionPlots';
 import EditDatasetDialog from './EditDatasetDialog';
 import EmbeddingCharts from './EmbeddingCharts';
 import GalleryCharts from './GalleryCharts';
 import HelpDialog from './HelpDialog';
+import JobResultsPanel from './JobResultsPanel';
 import MoreOptionsDialog from './MoreOptionsDialog';
 import SaveDatasetFilterDialog from './SaveDatasetFilterDialog';
 import SaveSetDialog from './SaveSetDialog';
@@ -142,9 +143,15 @@ class App extends PureComponent {
                             </div>
                             <div
                                 role="tabpanel"
-                                hidden={tab !== 'dot_plot'}
+                                hidden={tab !== 'distribution'}
                             >
-                                <DotPlots/>
+                                <DistributionPlots/>
+                            </div>
+                            <div
+                                role="tabpanel"
+                                hidden={tab !== 'results'}
+                            >
+                                <JobResultsPanel/>
                             </div>
                         </React.Fragment>}
 
