@@ -2,6 +2,7 @@ import {Divider, IconButton, Menu, Tooltip} from '@material-ui/core';
 import AppBar from '@material-ui/core/AppBar';
 import Button from '@material-ui/core/Button';
 import MenuItem from '@material-ui/core/MenuItem';
+import Popover from '@material-ui/core/Popover';
 import {withStyles} from '@material-ui/core/styles';
 import Tab from '@material-ui/core/Tab';
 import Tabs from '@material-ui/core/Tabs';
@@ -12,6 +13,7 @@ import Brightness2Icon from '@material-ui/icons/Brightness3';
 import HelpIcon from '@material-ui/icons/Help';
 import MoreVertIcon from '@material-ui/icons/MoreVert';
 import React from 'react';
+import ReactMarkdown from 'react-markdown';
 import {connect} from 'react-redux';
 import {
     DELETE_DATASET_DIALOG,
@@ -36,9 +38,7 @@ import {
     DEFAULT_LABEL_FONT_SIZE,
     DEFAULT_LABEL_STROKE_WIDTH,
     DEFAULT_MARKER_OPACITY,
-    DEFAULT_SHOW_AXIS,
     DEFAULT_SHOW_FOG,
-    DEFAULT_SHOW_LABELS,
     DEFAULT_UNSELECTED_MARKER_OPACITY
 } from "./reducers";
 
@@ -84,7 +84,6 @@ class AppHeader extends React.PureComponent {
             userMenuAnchorEl: null,
             moreMenuOpen: false,
             moreMenuAnchorEl: null,
-
         };
 
     }
@@ -137,7 +136,22 @@ class AppHeader extends React.PureComponent {
     };
 
     getLinkJson = () => {
-        const {chartOptions, combineDatasetFilters, primaryTraceKey, dataset, embeddings, searchTokens, datasetFilter, interpolator, markerOpacity, unselectedMarkerOpacity, dotPlotData} = this.props;
+        const {
+            chartOptions,
+            combineDatasetFilters,
+            primaryTraceKey,
+            dataset,
+            embeddingLabels,
+            distributionPlotOptions,
+            embeddings,
+            searchTokens,
+            datasetFilter,
+            interpolator,
+            markerOpacity,
+            pointSize,
+            unselectedMarkerOpacity,
+            distributionData
+        } = this.props;
 
         let json = {
             dataset: dataset.id,
@@ -158,13 +172,12 @@ class AppHeader extends React.PureComponent {
             json.camera = chartRef.scatterPlot.getCameraDef();
 
         }
-
-        let jsonChartOptions = {};
-        if (json.embeddings.length > 0) {
-            jsonChartOptions.activeEmbedding = primaryTraceKey;
+        if (primaryTraceKey != null) {
+            json.primaryTraceKey = primaryTraceKey;
         }
+        let jsonChartOptions = {};
+
         const defaultChartOptions = {
-            showLabels: DEFAULT_SHOW_LABELS, showAxis: DEFAULT_SHOW_AXIS,
             showFog: DEFAULT_SHOW_FOG, darkMode: DEFAULT_DARK_MODE,
             labelFontSize: DEFAULT_LABEL_FONT_SIZE,
             labelStrokeWidth: DEFAULT_LABEL_STROKE_WIDTH
@@ -176,10 +189,13 @@ class AppHeader extends React.PureComponent {
                 jsonChartOptions[key] = value;
             }
         }
-        if (Object.keys(jsonChartOptions).length > 0) {
-            json.chartOptions = jsonChartOptions;
+        if (pointSize !== 1) {
+            json.pointSize = pointSize;
         }
 
+        json.chartOptions = jsonChartOptions;
+
+
         if (searchTokens.length > 0) {
             json.q = searchTokens;
         }
@@ -207,16 +223,16 @@ class AppHeader extends React.PureComponent {
             json.unselectedMarkerOpacity = unselectedMarkerOpacity;
         }
 
-        if (dotPlotData && dotPlotData.length > 0) {
-            let sortOrder = {};
-            dotPlotData.forEach(data => {
-                sortOrder[data.name] = data.sortBy;
-            });
-            json.sort = sortOrder;
+        if (distributionData && distributionData.length > 0) {
+            json.distributionPlotOptions = distributionPlotOptions;
         }
         if (interpolator.name !== DEFAULT_INTERPOLATOR) {
             json.colorScheme = interpolator.name;
         }
+
+        if (embeddingLabels.length > 0) {
+            json.embeddingLabels = embeddingLabels;
+        }
         return json;
     };
 
@@ -228,6 +244,7 @@ class AppHeader extends React.PureComponent {
             savedDatasetState[this.props.dataset.id] = link;
             this.props.handleSavedDatasetState(savedDatasetState);
         }
+        this.props.handleTab('embedding'); // embedding won't render unless visible
         this.props.handleDataset(id);
     };
 
@@ -295,43 +312,62 @@ class AppHeader extends React.PureComponent {
         this.setState({moreMenuOpen: false});
     };
 
+    handleShowDatasetDetails = (event) => {
+        this.setState({datasetDetailsEl: event.currentTarget});
+    };
+    handleCloseDatasetDetails = (event) => {
+        this.setState({datasetDetailsEl: null});
+    };
+
     render() {
         const {
-            dataset, loadingApp, email, selection, classes, serverInfo, tab, user
+            dataset, loadingApp, jobResults, email, selection, classes, serverInfo, tab, user
         } = this.props;
+
+
+        const datasetDetailsOpen = Boolean(this.state.datasetDetailsEl);
+        const hasDatasetDetails = dataset != null && (dataset.title || dataset.description);
         const shape = dataset != null && dataset.shape != null ? dataset.shape : null;
         const hasSelection = dataset != null && shape != null && shape[0] > 0 && !isNaN(selection.count);
 
-        const showNewDataset = user != null && user.importer;
-        const showEditDeleteDataset = dataset !== null && dataset.owner;
-        const showMoreMenu = showNewDataset || dataset != null;
+        const showNewDataset = user != null && user.importer && !loadingApp.loading;
+        const showEditDeleteDataset = dataset !== null && dataset.owner && !loadingApp.loading;
+        const showMoreMenu = (showNewDataset || dataset != null) && !loadingApp.loading;
         const isSignedOut = !loadingApp.loading && email == null && serverInfo.clientId !== '';
         return (
-
             <AppBar position="fixed" color="default" className={classes.appBar}>
                 <Toolbar variant="dense">
-
-                    {/*{datasetChoices.length > 0 && <Select*/}
-                    {/*    style={{marginRight: 6}}*/}
-                    {/*    disableUnderline={true}*/}
-                    {/*    displayEmpty={true}*/}
-                    {/*    value={dataset == null ? '' : dataset.id}*/}
-                    {/*    onChange={this.handleDataset}*/}
-                    {/*    inputProps={{*/}
-                    {/*        name: 'dataset',*/}
-                    {/*        id: 'dataset-id',*/}
-                    {/*    }}*/}
-                    {/*>*/}
-                    {/*    <MenuItem key="" value="" disabled>*/}
-                    {/*        Choose a dataset*/}
-                    {/*    </MenuItem>*/}
-                    {/*    {datasetChoices.map(dataset => <MenuItem*/}
-                    {/*        key={dataset.id} value={dataset.id}>{dataset.name}</MenuItem>)}*/}
-                    {/*</Select>}*/}
-
+                    {dataset != null && <Popover
+                        id={"dataset-details"}
+                        open={datasetDetailsOpen}
+                        anchorEl={this.state.datasetDetailsEl}
+                        onClose={this.handleCloseDatasetDetails}
+                        anchorOrigin={{
+                            vertical: 'bottom',
+                            horizontal: 'center',
+                        }}
+                        transformOrigin={{
+                            vertical: 'top',
+                            horizontal: 'center',
+                        }}
+                    >
+                        <div style={{width: 500}}>
+                            {dataset.title && <Typography className={classes.typography}>
+                                {dataset.title}
+                            </Typography>}
+                            {dataset.description &&
+                            <ReactMarkdown linkTarget="_blank" children={dataset.description}/>}
+                        </div>
+                    </Popover>
+                    }
                     {dataset != null &&
-                    <Typography component={"h3"}>
-                        <Tooltip title={dataset.description||''}><b>{dataset.name}</b></Tooltip>
+                    <Typography
+                        aria-owns={this.state.datasetDetailsOpen ? 'dataset-details' : undefined}
+                        aria-haspopup="true"
+                        component={"h3"}>
+                        {hasDatasetDetails &&
+                        <Button onClick={this.handleShowDatasetDetails}><b>{dataset.name}</b></Button>}
+                        {!hasDatasetDetails && <b>{dataset.name}</b>}
                         <small>&nbsp;
                             {hasSelection && shape != null && intFormat(selection.count) + ' / '}
                             {shape != null && intFormat(shape[0]) + ' cells'}
@@ -345,11 +381,10 @@ class AppHeader extends React.PureComponent {
                         indicatorColor="primary"
                         textColor="primary"
                         onChange={this.handleTabChange}
-                        aria-label="view"
-
                     >
                         <AntTab value="embedding" label="Embeddings"/>
-                        <AntTab value="dot_plot" label="Dot Plot"/>
+                        <AntTab value="distribution" label="Distributions"/>
+                        {jobResults.length > 0 && <AntTab value="results" label="Results"/>}
                     </Tabs>}
 
                     <div className={"cirro-condensed"}>
@@ -357,10 +392,9 @@ class AppHeader extends React.PureComponent {
                         {/*<h3*/}
                         {/*    style={{display: 'inline', marginRight: 20}}>Cirro</h3>*/}
 
-
                     </div>
                     <div style={{marginLeft: 'auto'}}>
-                        {!isSignedOut && <DatasetSelector onChange={this.handleDataset}/>}
+                        {!loadingApp.loading && !isSignedOut && <DatasetSelector onChange={this.handleDataset}/>}
                         {showMoreMenu && <Tooltip title={'More'}>
                             <IconButton aria-label="Menu" aria-haspopup="true"
                                         onClick={this.handleMoreMenuOpen}>
@@ -389,7 +423,6 @@ class AppHeader extends React.PureComponent {
                             {dataset != null && <MenuItem onClick={this.copyLink}>Copy Link </MenuItem>}
                         </Menu>}
 
-
                         {<Tooltip title={"Toggle Light/Dark Theme"}>
                             <IconButton edge={false} className={this.props.chartOptions.darkMode ? 'cirro-active' : ''}
                                         aria-label="Toggle Theme" onClick={() => this.onDarkMode()}>
@@ -425,7 +458,6 @@ class AppHeader extends React.PureComponent {
                             <MenuItem onClick={this.handleLogout}>Sign Out</MenuItem>
                         </Menu>}
 
-
                         {isSignedOut && <Button style={{whiteSpace: 'nowrap'}} variant="outlined" color="primary"
                                                 onClick={this.props.handleLogin}>Sign In</Button>}
                     </div>
@@ -441,6 +473,7 @@ const mapStateToProps = state => {
         dataset: state.dataset,
         chartOptions: state.chartOptions,
         embeddings: state.embeddings,
+        embeddingLabels: state.embeddingLabels,
         searchTokens: state.searchTokens,
         primaryTraceKey: state.primaryTraceKey,
         binSummary: state.binSummary,
@@ -448,14 +481,16 @@ const mapStateToProps = state => {
         combineDatasetFilters: state.combineDatasetFilters,
         datasetFilter: state.datasetFilter,
         markerOpacity: state.markerOpacity,
+        pointSize: state.pointSize,
         unselectedMarkerOpacity: state.unselectedMarkerOpacity,
         savedDatasetState: state.savedDatasetState,
         datasetChoices: state.datasetChoices,
         dialog: state.dialog,
-        dotPlotData: state.dotPlotData,
+        distributionData: state.distributionData,
         email: state.email,
 
         interpolator: state.interpolator,
+        jobResults: state.jobResults,
         loading: state.loading,
         loadingApp: state.loadingApp,
 
