@@ -9,6 +9,7 @@ import Menu from '@material-ui/core/Menu';
 import MenuItem from '@material-ui/core/MenuItem';
 import TextField from '@material-ui/core/TextField';
 import ArrowDropDownIcon from '@material-ui/icons/ArrowDropDown';
+import natsort from 'natsort';
 import React from 'react';
 import {intFormat, numberFormat} from './formatters';
 
@@ -97,7 +98,17 @@ class CategoricalLegend extends React.PureComponent {
 
 
     render() {
-        const {scale, datasetFilter, name, featureSummary, maxHeight, globalFeatureSummary, nObs, nObsSelected, categoricalNames} = this.props;
+        const {
+            scale,
+            datasetFilter,
+            name,
+            featureSummary,
+            maxHeight,
+            globalFeatureSummary,
+            nObs,
+            nObsSelected,
+            categoricalNames
+        } = this.props;
         let clickEnabled = this.props.clickEnabled;
         const categoricalFilterValues = datasetFilter[name];
         const selectionSummary = featureSummary[name];
@@ -120,6 +131,19 @@ class CategoricalLegend extends React.PureComponent {
         // }
         const categories = globalDimensionSummary.categories;
         const renamedCategories = categoricalNames[name] || {};
+        const sorter = natsort({insensitive: true});
+        categories.sort((a, b) => {
+            let renamed1 = renamedCategories[a];
+            if (renamed1 != null) {
+                a = renamed1;
+            }
+            let renamed2 = renamedCategories[b];
+            if (renamed2 != null) {
+                b = renamed2;
+            }
+            return sorter(a, b);
+        });
+
         clickEnabled = clickEnabled && categories.length > 1;
         let style = {maxHeight: maxHeight, display: 'inline-block'};
         if (this.props.style) {
@@ -208,15 +232,25 @@ class CategoricalLegend extends React.PureComponent {
                     {categories.map((category, i) => {
 
                         const opacity = categoricalFilterValues == null || categoricalFilterValues.indexOf(category) !== -1 ? 1 : 0.4;
-                        // const fractionUnselected = unselectedCounts != null ? unselectedCounts[i] / unselectedTotal : null;
-                        // const unselectedSize = unselectedCounts == null ? 0 : fractionScale(fractionUnselected);
-                        // const unselectedTitle = unselectedCounts == null ? null : intFormat(unselectedCounts[i]) + ' / ' + intFormat(unselectedTotal) + (unselectedCounts[i] > 0 ? (' ( ' + numberFormat(100 * fractionUnselected) + '%)') : '');
-                        const count = globalDimensionSummary.counts[i];
-                        const selectedCount = selectedDimensionToCount[category] || 0;
-                        const fractionSelected = selectionSummary == null ? 0 : selectedCount / nObsSelected;
+                        // const fractionUnselected = unselectedCategoryCounts != null ? unselectedCategoryCounts[i] / unselectedTotal : null;
+                        // const unselectedSize = unselectedCategoryCounts == null ? 0 : fractionScale(fractionUnselected);
+                        // const unselectedTitle = unselectedCategoryCounts == null ? null : intFormat(unselectedCategoryCounts[i]) + ' / ' + intFormat(unselectedTotal) + (unselectedCategoryCounts[i] > 0 ? (' ( ' + numberFormat(100 * fractionUnselected) + '%)') : '');
+                        const categoryCount = globalDimensionSummary.counts[i];
+                        const selectedCategoryCount = selectedDimensionToCount[category] || 0;
+
+                        //            not-selected, selected
+                        // in category      a       b
+                        // not in category  c       d
+                        const a = categoryCount - selectedCategoryCount;
+                        const b = selectedCategoryCount;
+                        const c = nObs - nObsSelected - selectedCategoryCount;
+                        const d = nObsSelected - selectedCategoryCount;
+                        // const pValue = fisherTest(a, b, c, d);
+
+                        const fractionSelected = selectionSummary == null ? 0 : selectedCategoryCount / nObsSelected;
                         // const selectedSize = fractionScale(fractionSelected);
                         // const globalSize = fractionScale(count / nObs);
-                        const globalTitle = numberFormat(100 * count / nObs) + '%';
+                        const globalTitle = numberFormat(100 * categoryCount / nObs) + '%';
                         let categoryText = category;
                         let renamed = renamedCategories[category];
                         if (renamed !== undefined) {
@@ -253,7 +287,7 @@ class CategoricalLegend extends React.PureComponent {
 
                             <td>
                                 <Tooltip title={globalTitle}>
-                                    <span>{intFormat(count)}</span>
+                                    <span>{intFormat(categoryCount)}</span>
                                 </Tooltip>
 
                                 {/*<div*/}
@@ -278,7 +312,7 @@ class CategoricalLegend extends React.PureComponent {
                             </td>
                             {selectionSummary && <td>
                                 <Tooltip title={selectionTitle}>
-                                    <div>{intFormat(selectedCount)}</div>
+                                    <div>{intFormat(selectedCategoryCount)}</div>
                                 </Tooltip>
                                 {/*<div*/}
                                 {/*    title={selectionTitle}*/}
