@@ -1,33 +1,46 @@
-import {Tooltip, withStyles} from '@material-ui/core';
-import FormControl from '@material-ui/core/FormControl';
+import {Tooltip} from '@material-ui/core';
 import IconButton from '@material-ui/core/IconButton';
-import Input from '@material-ui/core/Input';
-import InputLabel from '@material-ui/core/InputLabel';
 import Menu from '@material-ui/core/Menu';
 import MenuItem from '@material-ui/core/MenuItem';
-import Select from '@material-ui/core/Select';
 import Typography from '@material-ui/core/Typography';
 import PhotoCameraIcon from '@material-ui/icons/PhotoCamera';
-import natsort from 'natsort';
 import React from 'react';
+import {CANVAS_FONT, SVG_FONT} from './ChartUtil';
 import {drawColorScheme} from './ColorSchemeLegend';
-import {numberFormat} from './formatters';
+import {numberFormat, numberFormat2f} from './formatters';
 import {drawSizeLegend} from './SizeLegend';
+import {stripTrailingZeros} from './util';
 
-const styles = theme => ({
-    root: {},
-    formControl: {
-        display: 'block',
-        margin: theme.spacing(1),
+export const CHIP_SIZE = 12;
+
+export function getNameWidth(array2d, context) {
+    let offsets = [];
+    let ncategories = 0;
+    if (array2d[0].length > 0) {
+        ncategories = array2d[0][0].name.length;
+        for (let i = 0; i < array2d[0][0].name.length; i++) {
+            offsets.push(0);
+        }
     }
-
-});
-
-let svgFont = '12px Helvetica,Arial,sans-serif';
-let canvasFont = '12px Roboto Condensed,Helvetica,Arial,sans-serif';
+    array2d.forEach(array => {
+        let name = array[0].name;
+        for (let i = 0; i < ncategories; i++) {
+            offsets[i] = Math.max(offsets[i], context.measureText(name[i]).width);
+        }
+    });
+    for (let i = 0; i < offsets.length; i++) {
+        // 4px, chip, 2px, text
+        offsets[i] += 6;
+        offsets[i] += CHIP_SIZE;
+    }
+    offsets[offsets.length - 1] += 4;
+    let xTotal = 0;
+    offsets.forEach(val => xTotal += val);
+    return {offsets: offsets, sum: xTotal};
+}
 
 
-class DotPlotCanvas extends React.PureComponent {
+export default class DotPlotCanvas extends React.PureComponent {
 
     constructor(props) {
         super(props);
@@ -37,10 +50,6 @@ class DotPlotCanvas extends React.PureComponent {
         this.state = {saveImageEl: null};
     }
 
-    onSortOrderChanged = (event) => {
-        this.props.onSortOrderChanged({name: this.props.data.name, value: event.target.value});
-    };
-
 
     redraw() {
 
@@ -56,28 +65,29 @@ class DotPlotCanvas extends React.PureComponent {
             let onMouseMove = (event) => {
                 const node = event.target;
                 const maxRadius = this.props.sizeScale.range()[1];
-                var rect = node.getBoundingClientRect();
+                const rect = node.getBoundingClientRect();
                 let xy = [event.clientX - rect.left - node.clientLeft, event.clientY - rect.top - node.clientTop];
                 // xy[0] /= devicePixelRatio;
                 // xy[1] /= devicePixelRatio;
                 const col = Math.floor((xy[0] - this.size.x) / (maxRadius * 2));
                 const row = Math.floor((xy[1]) / (maxRadius * 2));
 
-                if (col >= 0 && col < this.dotplot.values.length && row >= 0 && row < this.categories.length) {
+                if (col >= 0 && col < this.props.data[0].length && row >= 0 && row < this.props.data.length) {
                     this.tooltipElementRef.current.innerHTML = '';
-                    let featureDatum = this.dotplot.values[col];
-                    const mean = featureDatum.mean[this.categoryOrder[row]];
-                    const fractionExpressed = featureDatum.fractionExpressed[this.categoryOrder[row]];
-                    this.tooltipElementRef.current.innerHTML = 'mean: ' + numberFormat(mean) + ', % expressed: ' + numberFormat(100 * fractionExpressed);
+                    const array = this.props.data[row];
+                    const mean = array[col].mean;
+
+                    let meanFormatted = stripTrailingZeros(numberFormat2f(mean));
+                    let percentExpressed = stripTrailingZeros(numberFormat(array[col].percentExpressed));
+
+                    this.tooltipElementRef.current.innerHTML = 'mean: ' + meanFormatted + ', % expressed: ' + percentExpressed + ', ' + array[col].feature + ', ' + array[col].name.join(', ');
                 } else {
                     this.tooltipElementRef.current.innerHTML = '';
                 }
             };
             let onMouseOut = (event) => {
                 this.tooltipElementRef.current.innerHTML = '';
-
             };
-            // onMouseMove = throttle(onMouseMove);
             this.canvas = document.createElement('canvas');
             this.canvas.addEventListener("mousemove", onMouseMove);
             this.canvas.addEventListener("mouseout", onMouseOut);
@@ -92,89 +102,81 @@ class DotPlotCanvas extends React.PureComponent {
         canvas.height = height * devicePixelRatio;
         canvas.style.width = width + 'px';
         canvas.style.height = height + 'px';
-        context.font = canvasFont;
+        context.font = CANVAS_FONT;
 
         context
             .clearRect(0, 0, width * devicePixelRatio, height * devicePixelRatio);
         context.scale(devicePixelRatio, devicePixelRatio);
-        this.drawContext(context);
+        this.drawContext(context, this.size);
     }
 
-    drawContext(context) {
-        const renamedCategories = this.props.renamedCategories || {};
-        const dotplot = this.dotplot;
+    drawContext(context, size) {
+        const data2d = this.props.data;
         const colorScale = this.props.colorScale;
-        const features = this.features;
+        const categoryColorScales = this.props.categoryColorScales;
         const sizeScale = this.props.sizeScale;
-        const categories = this.categories;
-        const categoryOrder = this.categoryOrder;
+        const drawCircles = this.props.drawCircles;
         const textColor = this.props.textColor;
         const maxRadius = sizeScale.range()[1];
         let diameter = maxRadius * 2;
         // context.strokeStyle = gridColor;
         // context.lineWidth = gridThickness;
-        dotplot.values.forEach((datum, featureIndex) => { // each feature
-            for (let i = 0; i < datum.mean.length; i++) { // each category
-                const mean = datum.mean[categoryOrder[i]];
-                const frac = datum.fractionExpressed[categoryOrder[i]];
+
+        data2d.forEach((array, j) => { // each category
+            const ypix = j * diameter + (drawCircles ? maxRadius : 0);
+            for (let i = 0; i < array.length; i++) { // each feature
+                const mean = array[i].mean;
                 const color = colorScale(mean);
-                const xpix = featureIndex * diameter + maxRadius + this.size.x;
-                const ypix = i * diameter + maxRadius;
                 context.fillStyle = color;
                 context.beginPath();
-                context.arc(xpix, ypix, sizeScale(frac), 0, 2 * Math.PI);
+                if (drawCircles) {
+                    const xpix = i * diameter + maxRadius + size.x;
+                    const frac = array[i].percentExpressed;
+                    context.arc(xpix, ypix, sizeScale(frac), 0, 2 * Math.PI);
+                } else {
+                    const xpix = i * diameter + size.x;
+                    context.rect(xpix, ypix, diameter, diameter);
+                }
                 context.fill();
                 // context.stroke();
             }
         });
-        // context.lineWidth = 1;
-        context.textAlign = 'right';
-        context.fillStyle = textColor;
+
+        context.textAlign = 'left';
         context.textBaseline = 'middle';
 
-        for (let i = 0; i < categories.length; i++) {
-            let category = categories[categoryOrder[i]];
-            let newName = renamedCategories[category];
-            if (newName != null) {
-                category = newName;
-            }
+        data2d.forEach((array, i) => { // categories
+            let name = array[0].name;
             const pix = i * diameter + maxRadius;
-            context.fillText(category, this.size.x - 4, pix);
-        }
+            for (let j = 0; j < name.length; j++) {
+                // 4px, chip, 2px, text
+                let xpixstart = size.xoffsets[j - 1] || 0;
+                if (xpixstart > 0) {
+                    xpixstart += 4;
+                }
+                const categoryColorScale = categoryColorScales[j];
+                context.fillStyle = categoryColorScale(array[0].categories[j]);
+                context.beginPath();
+                context.rect(xpixstart, pix - maxRadius / 2 - 3, CHIP_SIZE, CHIP_SIZE);
+                context.fill();
+                context.stroke();
+                context.fillStyle = textColor;
+                context.fillText(name[j], xpixstart + 2 + CHIP_SIZE, pix);
+            }
+        });
         context.textAlign = 'right';
         context.textBaseline = 'top';
-        for (let i = 0; i < features.length; i++) {
-            const text = features[i];
+
+        data2d[0].forEach((item, i) => { // features
+            const text = item.feature;
             const pix = i * diameter;
             context.save();
-            context.translate(this.size.x + pix + 4, this.size.height);
+            context.translate(size.x + pix + 4, size.height);
             context.rotate(-Math.PI / 2);
             context.fillText(text, 0, 0);
             context.restore();
-
-        }
-
-        // context.strokeStyle = gridColor;
-        // context.lineWidth = gridThickness;
-        //
-        //
-        // for (let i = 0; i < categories.length; i++) {
-        //     const ypix = i * diameter;
-        //     context.beginPath();
-        //     context.moveTo(this.size.x + 2, ypix);
-        //     context.lineTo(width, ypix);
-        //     context.stroke();
-        // }
-        // for (let i = 0; i < features.length; i++) {
-        //     const xpix = i * diameter + this.size.x + 2;
-        //     context.beginPath();
-        //     context.moveTo(xpix, 0);
-        //     context.lineTo(xpix, dotplotHeight);
-        //     context.stroke();
-        // }
-
+        });
         context.setTransform(1, 0, 0, 1, 0, 0);
-
     }
 
 
@@ -188,96 +190,25 @@ class DotPlotCanvas extends React.PureComponent {
 
     getSize(context) {
         let maxFeatureWidth = 0;
-        const renamedCategories = this.props.renamedCategories || {};
-        this.dotplot.values.forEach(datum => {
-            maxFeatureWidth = Math.max(maxFeatureWidth, context.measureText(datum.name).width);
+        const array2d = this.props.data;
+        array2d[0].forEach(item => {
+            maxFeatureWidth = Math.max(maxFeatureWidth, context.measureText(item.feature).width);
         });
         maxFeatureWidth += 4;
-        let xoffset = 0;
-        this.categories.forEach(category => {
-            let renamed = renamedCategories[category];
-            if (renamed != null) {
-                category = renamed;
-            }
-            xoffset = Math.max(xoffset, context.measureText(category).width);
-        });
-        xoffset += 4;
+
+        const nameWidth = getNameWidth(array2d, context);
         const maxRadius = this.props.sizeScale.range()[1];
         const diameter = maxRadius * 2;
-        const height = this.categories.length * diameter + 4;
-        const width = this.features.length * diameter + 4;
-        return {x: xoffset, y: maxFeatureWidth, width: width, height: height};
+        const height = array2d.length * diameter + 4;
+        const width = array2d[0].length * diameter + 4;
+        return {xoffsets: nameWidth.offsets, x: nameWidth.sum, y: maxFeatureWidth, width: width, height: height};
     }
 
     update() {
-
-        let dotplot = Object.assign({}, this.props.data);
-        const renamedCategories = this.props.renamedCategories || {};
-        if (dotplot != null && dotplot.selection) {
-            dotplot = dotplot.selection;
-        }
-        this.dotplot = dotplot;
-        const categories = dotplot.categories || [''];
-
-        this.categories = categories;
-        const features = dotplot.values.map(feature => feature.name);
-
-        if (dotplot.sortBy == null) {
-            dotplot.sortBy = dotplot.name;
-        }
-        let categoryOrder = [];
-        for (let i = 0; i < categories.length; i++) {
-            categoryOrder.push(i);
-        }
-        if (dotplot.sortBy !== dotplot.name) { // sort by feature
-            let sortByDatum;
-            for (let i = 0; i < dotplot.values.length; i++) {
-                if (dotplot.values[i].name === dotplot.sortBy) {
-                    sortByDatum = dotplot.values[i];
-                    break;
-                }
-            }
-            if (sortByDatum) {
-                categoryOrder.sort((a, b) => {
-                    let val1 = sortByDatum.mean[a];
-                    let val2 = sortByDatum.mean[b];
-                    let c = val1 === val2 ? 0 : (val1 > val2 ? -1 : 1);
-                    if (c === 0) {
-                        val1 = sortByDatum.fractionExpressed[a];
-                        val2 = sortByDatum.fractionExpressed[b];
-                        c = val1 === val2 ? 0 : (val1 > val2 ? -1 : 1);
-                    }
-                    return c;
-                });
-
-            }
-        } else { // sort by category
-            if (Object.keys(renamedCategories).length > 0) {
-                const sorter = natsort();
-                categoryOrder.sort((a, b) => {
-                    let val1 = categories[a];
-                    let renamed1 = renamedCategories[val1];
-                    if (renamed1 != null) {
-                        val1 = renamed1;
-                    }
-                    let val2 = categories[b];
-                    let renamed2 = renamedCategories[val2];
-                    if (renamed2 != null) {
-                        val2 = renamed2;
-                    }
-                    return sorter(val1, val2);
-                });
-            }
-
-        }
-
         const canvas = document.createElement('canvas');
         const context = canvas.getContext('2d');
-        context.font = canvasFont;
-        this.features = features;
+        context.font = CANVAS_FONT;
         this.size = this.getSize(context);
-        this.categoryOrder = categoryOrder;
-
     }
 
     handleSaveImageMenu = (event) => {
@@ -287,7 +218,6 @@ class DotPlotCanvas extends React.PureComponent {
         this.setState({saveImageEl: null});
     };
 
-
     handleSaveImage = (format) => {
         this.setState({saveImageEl: null});
         let context;
@@ -295,45 +225,41 @@ class DotPlotCanvas extends React.PureComponent {
         let canvas;
         if (format === 'svg') {
             context = new window.C2S(10, 10);
-            context.font = svgFont;
+            context.font = SVG_FONT;
         } else {
             canvas = document.createElement('canvas');
             context = canvas.getContext('2d');
-            context.font = canvasFont;
+            context.font = CANVAS_FONT;
         }
 
         const size = this.getSize(context);
-
-        const colorScaleHeight = 40;
+        const colorScaleHeight = 15 + 20;
         const sizeScaleHeight = 40;
         const height = size.height + size.y + colorScaleHeight + sizeScaleHeight + 10;
         const width = Math.max(200, size.width + size.x);
         let scale = 1;
         if (format === 'svg') {
             context = new window.C2S(width, height);
-            context.font = svgFont;
+            context.font = SVG_FONT;
         } else {
             canvas.width = width * window.devicePixelRatio;
             canvas.height = height * window.devicePixelRatio;
             context = canvas.getContext('2d');
             scale = window.devicePixelRatio;
-            context.scale(window.devicePixelRatio, window.devicePixelRatio);
-            context.font = canvasFont;
+            context.scale(scale, scale);
+            context.font = CANVAS_FONT;
         }
         const textColor = this.props.textColor;
-        if (textColor === 'white') {
-            context.fillStyle = 'black';
-            context.fillRect(0, 0, width, height);
-        }
-        this.drawContext(context);
+        context.fillStyle = textColor === 'white' ? 'black' : 'white';
+        context.fillRect(0, 0, width, height);
+        this.drawContext(context, size);
 
         // if (format !== 'svg') {
         //     context.scale(window.devicePixelRatio, window.devicePixelRatio);
         // }
 
-        context.translate(scale * 10, scale * (size.height + size.y + 4));
-        drawColorScheme(context, 150, colorScaleHeight, this.props.colorScale, true, 10, textColor);
-
+        context.translate(4, scale * (size.height + size.y + 4));
+        drawColorScheme(context, this.props.colorScale, textColor);
         context.translate(-10, (colorScaleHeight + 4));
 
         drawSizeLegend(context, this.props.sizeScale, 3, 150, 20, textColor);
@@ -348,10 +274,10 @@ class DotPlotCanvas extends React.PureComponent {
             let blob = new Blob([svg], {
                 type: 'text/plain;charset=utf-8'
             });
-            window.saveAs(blob, this.dotplot.name + '.svg');
+            window.saveAs(blob, this.props.data[0][0].dimension + '.svg');
         } else {
             canvas.toBlob(blob => {
-                window.saveAs(blob, this.dotplot.name + '.png', true);
+                window.saveAs(blob, this.props.data[0][0].dimension + '.png', true);
             });
         }
     };
@@ -359,13 +285,13 @@ class DotPlotCanvas extends React.PureComponent {
     render() {
         this.update();
         const {saveImageEl} = this.state;
-        const dotplot = this.dotplot;
-        const features = this.features;
-        const sortChoices = [dotplot.name].concat(features);
+        const array2d = this.props.data;
+        const dimension = array2d[0][0].dimension;
+
         return (<div style={{position: 'relative'}}>
             <div>
                 <Typography style={{display: 'inline-block'}} component={"h4"}
-                            color="textPrimary">{dotplot.name}{this.props.subtitle &&
+                            color="textPrimary">{dimension}{this.props.subtitle &&
                 <small>({this.props.subtitle})</small>}</Typography>
                 <Tooltip title={"Save Image"}>
                     <IconButton aria-controls="save-image-menu" aria-haspopup="true" edge={false}
@@ -397,19 +323,6 @@ class DotPlotCanvas extends React.PureComponent {
                     textOverflow: 'ellipsis'
                 }}></Typography>
 
-                {this.props.onSortOrderChanged ? <FormControl className={this.props.classes.formControl}>
-                    <InputLabel shrink={true}>Sort By</InputLabel>
-                    <Select
-
-                        input={<Input size={"small"}/>}
-                        onChange={this.onSortOrderChanged}
-                        value={dotplot.sortBy}
-                    >
-                        {sortChoices.map(item => (
-                            <MenuItem key={item} value={item}>{item}</MenuItem>
-                        ))}
-                    </Select>
-                </FormControl> : null}
             </div>
             <div ref={this.divRef}></div>
 
@@ -418,6 +331,5 @@ class DotPlotCanvas extends React.PureComponent {
     }
 }
 
-export default withStyles(styles)(DotPlotCanvas);
 
 
