@@ -12,15 +12,14 @@ import {
     handleDimensionFilterUpdated,
     handleDomainChange,
     handleMeasureFilterUpdated,
-    MORE_OPTIONS_DIALOG,
-    setChartOptions,
-    setDialog,
-    setPrimaryChartSize
+    setChartOptions
 } from './actions';
 import CategoricalLegend from './CategoricalLegend';
 import ColorSchemeLegendWrapper from './ColorSchemeLegendWrapper';
 import ImageChart from './ImageChart';
+import MetaEmbedding from './MetaEmbedding';
 import ScatterChartThree from './ScatterChartThree';
+import {splitSearchTokens} from './util';
 
 class EmbeddingChart extends React.PureComponent {
 
@@ -29,11 +28,6 @@ class EmbeddingChart extends React.PureComponent {
         this.state = {showDetails: true};
     }
 
-    setChartRef = (element) => {
-        this.props.chartOptions.ref = element;
-
-    };
-
     componentDidUpdate(prevProps, prevState, snapshot) {
         if (prevProps.traceInfo.name !== this.props.traceInfo.name) {
             this.setState({showDetails: true});
@@ -47,14 +41,39 @@ class EmbeddingChart extends React.PureComponent {
 
     render() {
         const {
-            onChartOptions, onMoreOptions, onDomain, onDimensionFilterUpdated,
-            onColorChange, onNameChange, onMeasureFilterUpdated, onSelect, onDeselect, onGallery,
-            traceInfo, selection, datasetFilter, chartOptions, featureSummary, markerOpacity, unselectedMarkerOpacity, pointSize, globalFeatureSummary, shape, nObsSelected, categoricalNames, primaryChartSize
+            cachedData,
+            categoricalNames,
+            chartOptions,
+            dataset,
+            datasetFilter,
+            embeddingLabels,
+            featureSummary,
+            globalFeatureSummary,
+            markerOpacity,
+            nObsSelected,
+            onChartOptions,
+            onColorChange,
+            onDeselect,
+            onDimensionFilterUpdated,
+            onDomain,
+            onGallery,
+            onMeasureFilterUpdated,
+            onNameChange,
+            onSelect,
+            pointSize,
+            primaryChartSize,
+            searchTokens,
+            selection,
+            shape,
+            traceInfo,
+            unselectedMarkerOpacity
         } = this.props;
+
+        const activeEmbeddingLabels = splitSearchTokens(searchTokens).obsCat.filter(item => embeddingLabels.indexOf(item) !== -1);
         const displayName = traceInfo.name === '__count' ? '' : traceInfo.name;
 
         return (
-            <div  style={{position: 'relative'}}>
+            <div style={{position: 'relative'}}>
                 <Box color="text.primary" style={{
                     marginTop: '3.2px',
                     position: 'absolute',
@@ -112,8 +131,10 @@ class EmbeddingChart extends React.PureComponent {
                 </Box>
 
 
-                {!traceInfo.isImage &&
+                {traceInfo.type === 'scatter' &&
                 <ScatterChartThree traceInfo={traceInfo}
+                                   cachedData={cachedData}
+                                   obsCat={activeEmbeddingLabels}
                                    chartSize={primaryChartSize}
                                    setChartOptions={onChartOptions}
                                    chartOptions={chartOptions}
@@ -126,13 +147,24 @@ class EmbeddingChart extends React.PureComponent {
                                    unselectedMarkerOpacity={unselectedMarkerOpacity}
                                    color={traceInfo.colors}
                                    onGallery={onGallery}
-                                   onMoreOptions={onMoreOptions}
-                                   ref={this.setChartRef}
 
                 />}
 
+                {traceInfo.type === 'meta_image' &&
+                <MetaEmbedding traceInfo={traceInfo}
+                               cachedData={cachedData}
+                               chartSize={primaryChartSize}
+                               setChartOptions={onChartOptions}
+                               chartOptions={chartOptions}
+                               dataset={dataset}
+                               categoricalNames={categoricalNames}
+                               markerOpacity={markerOpacity}
+                               onGallery={onGallery}
 
-                {traceInfo.isImage && <ImageChart
+                />}
+                {traceInfo.type === 'image' && <ImageChart
+                    cachedData={cachedData}
+                    obsCat={activeEmbeddingLabels}
                     setChartOptions={onChartOptions}
                     chartOptions={chartOptions}
                     style={{display: 'inline-block'}}
@@ -147,8 +179,6 @@ class EmbeddingChart extends React.PureComponent {
                     onDeselect={onDeselect}
                     onSelected={onSelect}
                     onGallery={onGallery}
-                    onMoreOptions={onMoreOptions}
-                    ref={this.setChartRef}
                 />}
 
 
@@ -159,21 +189,24 @@ class EmbeddingChart extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
-        chartOptions: state.chartOptions,
-        embeddingChartSize: state.embeddingChartSize,
-        pointSize: state.pointSize,
-        embeddingData: state.embeddingData,
-        primaryChartSize: state.primaryChartSize,
-        markerOpacity: state.markerOpacity,
-        unselectedMarkerOpacity: state.unselectedMarkerOpacity,
-        primaryTraceKey: state.primaryTraceKey,
+        cachedData: state.cachedData,
         categoricalNames: state.categoricalNames,
-        globalFeatureSummary: state.globalFeatureSummary,
-        featureSummary: state.featureSummary,
-        shape: state.dataset.shape,
-        nObsSelected: state.selection.count,
+        chartOptions: state.chartOptions,
         dataset: state.dataset,
-        datasetFilter: state.datasetFilter
+        datasetFilter: state.datasetFilter,
+        embeddingChartSize: state.embeddingChartSize,
+        embeddingData: state.embeddingData,
+        embeddingLabels: state.embeddingLabels,
+        featureSummary: state.featureSummary,
+        globalFeatureSummary: state.globalFeatureSummary,
+        markerOpacity: state.markerOpacity,
+        nObsSelected: state.selection.count,
+        pointSize: state.pointSize,
+        primaryChartSize: state.primaryChartSize,
+        primaryTraceKey: state.primaryTraceKey,
+        shape: state.dataset.shape,
+        searchTokens: state.searchTokens,
+        unselectedMarkerOpacity: state.unselectedMarkerOpacity,
     };
 };
 const mapDispatchToProps = dispatch => {
@@ -181,12 +214,6 @@ const mapDispatchToProps = dispatch => {
         onChartOptions: (options) => {
             dispatch(setChartOptions(options));
         },
-        onMoreOptions: () => {
-            dispatch(setDialog(MORE_OPTIONS_DIALOG));
-        },
-        onPrimaryChartSize: value => {
-            dispatch(setPrimaryChartSize(value));
-        },
         onDomain: (value) => {
             dispatch(handleDomainChange(value));
         },
