@@ -8,6 +8,7 @@ import {sortableContainer, sortableElement} from 'react-sortable-hoc';
 import {getTraceKey, setEmbeddingData, setPrimaryTraceKey} from './actions';
 import GalleryImage from './GalleryImage';
 import {createScatterPlot} from './ThreeUtil';
+import {splitSearchTokens} from './util';
 
 function createContainer(chartSize) {
     const containerElement = document.createElement('div');
@@ -23,7 +24,7 @@ class GalleryCharts extends React.PureComponent {
         super(props);
         this.containerElement = createContainer(this.props.chartSize);
         document.body.appendChild(this.containerElement);
-        this.scatterPlot = createScatterPlot(this.containerElement, window.ApplePaySession, false, false);
+        this.scatterPlot = createScatterPlot(this.containerElement, window.ApplePaySession != null, false, false);
         this.emptySet = new Set();
 
     }
@@ -41,7 +42,20 @@ class GalleryCharts extends React.PureComponent {
 
 
     render() {
-        let {categoricalNames, chartSize, embeddingData, primaryChartSize, markerOpacity, unselectedMarkerOpacity, pointSize, chartOptions, selection} = this.props;
+        let {
+            cachedData,
+            categoricalNames,
+            chartSize,
+            chartOptions,
+            embeddingData,
+            embeddingLabels,
+            markerOpacity,
+            pointSize,
+            primaryChartSize,
+            searchTokens,
+            selection,
+            unselectedMarkerOpacity
+        } = this.props;
         if (this.containerElement.style.width !== this.props.chartSize + 'px') {
             document.body.removeChild(this.containerElement);
             this.containerElement = createContainer(this.props.chartSize);
@@ -56,12 +70,13 @@ class GalleryCharts extends React.PureComponent {
             }
         }
         galleryTraces.sort((a, b) => a.sortIndex - b.sortIndex);
-
+        const obsCat = splitSearchTokens(searchTokens).obsCat.filter(item => embeddingLabels.indexOf(item) !== -1);
         // const DragHandle = sortableHandle(() => <span>::</span>);
 
         const SortableItem = sortableElement(({traceInfo}) => <GalleryImage
             traceInfo={traceInfo}
-            color={traceInfo.colors}
+            obsCat={obsCat}
+            cachedData={cachedData}
             scatterPlot={this.scatterPlot}
             markerOpacity={markerOpacity}
             chartOptions={chartOptions}
@@ -98,16 +113,19 @@ class GalleryCharts extends React.PureComponent {
 
 const mapStateToProps = state => {
     return {
+        cachedData: state.cachedData,
         categoricalNames: state.categoricalNames,
-        embeddingData: state.embeddingData,
-        markerOpacity: state.markerOpacity,
-        unselectedMarkerOpacity: state.unselectedMarkerOpacity,
-        selection: state.selection,
-        primaryTraceKey: state.primaryTraceKey,
-        pointSize: state.pointSize,
+        chartOptions: state.chartOptions,
         chartSize: state.chartSize,
+        embeddingData: state.embeddingData,
+        embeddingLabels: state.embeddingLabels,
+        markerOpacity: state.markerOpacity,
+        pointSize: state.pointSize,
         primaryChartSize: state.primaryChartSize,
-        chartOptions: state.chartOptions
+        primaryTraceKey: state.primaryTraceKey,
+        searchTokens: state.searchTokens,
+        selection: state.selection,
+        unselectedMarkerOpacity: state.unselectedMarkerOpacity,
     };
 };
 const mapDispatchToProps = dispatch => {
