@@ -2,11 +2,17 @@ import {InputLabel} from '@material-ui/core';
 import MenuItem from '@material-ui/core/MenuItem';
 import Select from '@material-ui/core/Select';
 import TextField from '@material-ui/core/TextField';
+import {debounce} from 'lodash';
 import React from 'react';
 
 
 class MeasureFilter extends React.PureComponent {
 
+    constructor(props) {
+        super(props);
+        this.handleValueUpdate = debounce(this.handleValueUpdate, 500);
+    }
+
     getFilter() {
         let filter = this.props.datasetFilter[this.props.name];
         if (filter == null) {
@@ -16,20 +22,18 @@ class MeasureFilter extends React.PureComponent {
         return filter;
     }
 
+    handleValueUpdate = () => {
+        const filter = this.getFilter();
+        let value = parseFloat(filter.uiValue);
+        this.props.handleUpdate({name: this.props.name, value: value, update: true});
+
+    };
+
     handleOperationChanged = (event) => {
         const operation = event.target.value;
         this.props.handleUpdate({name: this.props.name, operation: operation, update: true});
     };
 
-
-    handleValueKeyPress = (event) => {
-        if (event.key === 'Enter') {
-            const filter = this.getFilter();
-            let value = parseFloat(filter.uiValue);
-            this.props.handleUpdate({name: this.props.name, value: value, update: true});
-        }
-    };
-
     handleValueChange = (event) => {
         const filter = this.getFilter();
         filter.uiValue = event.target.value;
@@ -39,6 +43,7 @@ class MeasureFilter extends React.PureComponent {
             value: filter.value,
             update: false
         });
+        this.handleValueUpdate();
     };
 
     render() {
@@ -50,7 +55,6 @@ class MeasureFilter extends React.PureComponent {
 
             <div style={{display: 'flex'}}>
                 <InputLabel shrink={true} id={id + '_label'}>Filter</InputLabel>
-
                 <Select
                     labelId={id + '_label'}
                     id={id}
@@ -67,8 +71,8 @@ class MeasureFilter extends React.PureComponent {
                     <MenuItem value={"!="}>{"!="}</MenuItem>
                 </Select>
 
-                <TextField onKeyPress={this.handleValueKeyPress}
-                           onChange={this.handleValueChange} value={filter.uiValue} style={{maxWidth: 60}}/>
+                <TextField
+                    onChange={this.handleValueChange} value={filter.uiValue} style={{maxWidth: 60}}/>
 
             </div>
         );
