@@ -7,9 +7,15 @@ import {Vector3, Vector4} from 'three';
 import {getEmbeddingKey} from './actions';
 import ChartToolbar from './ChartToolbar';
 import {saveImage} from './ChartUtil';
-import {numberFormat} from './formatters';
-import {createScatterPlot, getCategoryLabelsPositions, POINT_VISUALIZER_ID, updateScatterChart} from './ThreeUtil';
-import {isPointInside} from './util';
+import {numberFormat2f} from './formatters';
+import {
+    createScatterPlot,
+    getCategoryLabelsPositions,
+    getLabels,
+    POINT_VISUALIZER_ID,
+    updateScatterChart
+} from './ThreeUtil';
+import {indexSort, isPointInside} from './util';
 
 function clamp(x, min_v, max_v) {
     return Math.min(Math.max(x, min_v), max_v);
@@ -24,6 +30,29 @@ function mix(x, y, a) {
     return x * (1.0 - a) + y * a;
 }
 
+export function drawLabels(context, labels, positions, chartOptions, chartSize, camera) {
+    const pos = new Vector3();
+    context.fillStyle = chartOptions.darkMode ? 'white' : 'black';
+    context.strokeStyle = chartOptions.darkMode ? 'rgba(0,0,0,0.9)' : 'rgba(255,255,255,0.9)';
+    context.lineWidth = chartOptions.labelStrokeWidth;
+    context.textAlign = 'center';
+    context.textBaseline = "middle";
+    const width = chartSize.width;
+    const height = chartSize.height;
+    const widthHalf = width / 2;
+    const heightHalf = height / 2;
+    for (let i = 0, k = 0; i < labels.length; i++, k += 3) {
+        pos.x = positions[k];
+        pos.y = positions[k + 1];
+        pos.z = positions[k + 2];
+        pos.project(camera);
+        pos.x = (pos.x * widthHalf) + widthHalf;
+        pos.y = -(pos.y * heightHalf) + heightHalf;
+
+        context.strokeText(labels[i], pos.x, pos.y);
+        context.fillText(labels[i], pos.x, pos.y);
+    }
+}
 
 export function getVisualizer(scatterPlot, id) {
     for (let i = 0; i < scatterPlot.visualizers.length; i++) {
@@ -68,12 +97,14 @@ class ScatterChartThree extends React.PureComponent {
         this.tooltipElementRef = React.createRef();
         this.scatterPlot = null;
         this.lastHoverIndex = -1;
+        this.state = {forceUpdate: false};
     }
 
     componentDidUpdate(prevProps, prevState, snapshot) {
         if (prevProps.chartSize !== this.props.chartSize) {
             this.scatterPlot.resize();
         }
+        this.init();
         this.draw();
     }
 
@@ -88,6 +119,7 @@ class ScatterChartThree extends React.PureComponent {
         }
     }
 
+
     componentWillUnmount() {
         // if (this.scatterGL != null) {
         //     this.scatterGL.dispose();
@@ -106,11 +138,18 @@ class ScatterChartThree extends React.PureComponent {
     }
 
     drawContext(context, chartSize, format) {
-        const {traceInfo, markerOpacity, unselectedMarkerOpacity, selection, categoricalNames, chartOptions} = this.props;
-        const showLabels = this.props.chartOptions.showLabels && traceInfo.isCategorical;
-
+        const {
+            obsCat,
+            cachedData,
+            traceInfo,
+            markerOpacity,
+            unselectedMarkerOpacity,
+            selection,
+            categoricalNames,
+            chartOptions
+        } = this.props;
         const pointSize = this.calculatePointSize(traceInfo);
-        let scaleFactor = this.props.pointSize;
+        const scaleFactor = this.props.pointSize;
         const PI2 = 2 * Math.PI;
         const colors = traceInfo.colors;
         const positions = traceInfo.positions;
@@ -125,11 +164,11 @@ class ScatterChartThree extends React.PureComponent {
         const heightHalf = height / 2;
         const colorScale = scaleLinear().domain([0, 1]).range([0, 255]);
         const npoints = traceInfo.npoints;
-
         const is3d = traceInfo.dimensions === 3;
-        let outputPointSize = 0;
+        let outputPointSize;
         let fog = this.scatterPlot.scene.fog;
         let spriteVisualizer = getVisualizer(this.scatterPlot, POINT_VISUALIZER_ID);
+
         if (!is3d) {
             const PI = 3.1415926535897932384626433832795;
             const minScale = 0.1;  // minimum scaling factor
@@ -143,35 +182,45 @@ class ScatterChartThree extends React.PureComponent {
                 1. + 2. / PI * (maxScale - 1.) * Math.atan(inSpeed * (zoom - 1.));
             outputPointSize = pointSize * scale;
         }
+        let gl_PointSize = (outputPointSize * scaleFactor) / 4;
         const pos = new Vector3();
         let cameraSpacePos = new Vector4();
-        // vec4 cameraSpacePos = modelViewMatrix * vec4(position, 1.0);
-        // Project vertex in camera-space to screen coordinates using the camera's
-        // projection matrix.
-        // gl_Position = projectionMatrix * cameraSpacePos;
-
-
         let object = spriteVisualizer.points;
         let modelViewMatrix = object.modelViewMatrix.clone();
         modelViewMatrix.multiplyMatrices(camera.matrixWorldInverse, object.matrixWorld);
-        let gl_PointSize = (outputPointSize * scaleFactor) / 4;
         const showFog = chartOptions.showFog;
         const isSelectionEmpty = selection.size === 0;
-        for (let i = 0, j = 0, k = 0; i < npoints; i++, j += 4, k += 3) {
-            const isSelected = isSelectionEmpty || selection.has(i);
-            pos.x = positions[k];
-            pos.y = positions[k + 1];
-            pos.z = positions[k + 2];
+        let pointOrder;
+        if (is3d) {
+            pointOrder = new Uint32Array(npoints);
+            for (let i = 0; i < npoints; i++) {
+                pointOrder[i] = i;
+            }
+        } else {
+            const z = new Float32Array(npoints);
+            for (let i = 0; i < npoints; i++) {
+                z[i] = positions[i * 3 + 2];
+            }
+            pointOrder = indexSort(z, true);
+        }
+        for (let j = 0; j < npoints; j++) {
+            const index = pointOrder[j];
+            const positionIndex = index * 3;
+            const colorIndex = index * 4;
+            const isSelected = isSelectionEmpty || selection.has(index);
+            pos.x = positions[positionIndex];
+            pos.y = positions[positionIndex + 1];
+            pos.z = positions[positionIndex + 2];
             pos.project(camera);
 
-            let r = colors[j];
-            let g = colors[j + 1];
-            let b = colors[j + 2];
+            let r = colors[colorIndex];
+            let g = colors[colorIndex + 1];
+            let b = colors[colorIndex + 2];
             let a = isSelected ? markerOpacity : unselectedMarkerOpacity;
             if (is3d) {
-                cameraSpacePos.x = positions[k];
-                cameraSpacePos.y = positions[k + 1];
-                cameraSpacePos.z = positions[k + 2];
+                cameraSpacePos.x = positions[positionIndex];
+                cameraSpacePos.y = positions[positionIndex + 1];
+                cameraSpacePos.z = positions[positionIndex + 2];
                 cameraSpacePos.w = 1;
                 cameraSpacePos.applyMatrix4(modelViewMatrix);
                 outputPointSize = -pointSize / cameraSpacePos.z;
@@ -193,31 +242,15 @@ class ScatterChartThree extends React.PureComponent {
 
             context.fillStyle = 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
             context.beginPath();
-
             context.arc(pos.x, pos.y, gl_PointSize, 0, PI2);
             context.closePath();
             context.fill();
         }
-        if (showLabels) {
-            context.fillStyle = chartOptions.darkMode ? 'white' : 'black';
-            context.strokeStyle = chartOptions.darkMode ? 'rgba(0,0,0,0.9)' : 'rgba(255,255,255,0.9)';
-            context.strokeWidth = chartOptions.labelStrokeWidth;
+        if (obsCat.length > 0) {
+            const labelsPositions = getCategoryLabelsPositions(traceInfo.embedding, obsCat, cachedData);
             let font = format === 'svg' ? 'serif' : 'Roboto Condensed';
-            context.font = 'bold ' + this.props.chartOptions.labelFontSize + 'px ' + font;
-            const labelsPositions = getCategoryLabelsPositions(traceInfo, categoricalNames);
-            context.textAlign = 'center';
-            context.textBaseline = "middle";
-            for (let i = 0, k = 0; i < labelsPositions.labels.length; i++, k += 3) {
-                pos.x = labelsPositions.positions[k];
-                pos.y = labelsPositions.positions[k + 1];
-                pos.z = labelsPositions.positions[k + 2];
-                pos.project(camera);
-                pos.x = (pos.x * widthHalf) + widthHalf;
-                pos.y = -(pos.y * heightHalf) + heightHalf;
-
-                context.strokeText(labelsPositions.labels[i], pos.x, pos.y);
-                context.fillText(labelsPositions.labels[i], pos.x, pos.y);
-            }
+            context.font = 'bold ' + chartOptions.labelFontSize + 'px ' + font;
+            drawLabels(context, getLabels(obsCat, labelsPositions.labels, categoricalNames), labelsPositions.positions, chartOptions, chartSize, camera);
         }
     }
 
@@ -241,10 +274,6 @@ class ScatterChartThree extends React.PureComponent {
         saveImage(traceInfo, chartSize, bind(this.drawContext, this), format);
     };
 
-    onMoreOptions = () => {
-        this.props.onMoreOptions();
-    };
-
 
     onEditSelection = () => {
         this.props.chartOptions.editSelection = !this.props.chartOptions.editSelection;
@@ -252,16 +281,11 @@ class ScatterChartThree extends React.PureComponent {
     };
 
 
-    onShowLabels = () => {
-        this.props.chartOptions.showLabels = !this.props.chartOptions.showLabels;
-        this.props.setChartOptions(this.props.chartOptions);
+    resetCamera = () => {
+        this.scatterPlot.resetZoom();
     };
 
 
-    onGallery = () => {
-        this.props.onGallery();
-    };
-
     onShowAxis = () => {
         const axes = this.scatterPlot.scene.getObjectByName('axes');
         this.props.chartOptions.showAxis = !this.props.chartOptions.showAxis;
@@ -308,10 +332,9 @@ class ScatterChartThree extends React.PureComponent {
 
 
     init() {
-
         if (this.scatterPlot == null) {
             const containerElement = this.containerElementRef.current;
-            this.scatterPlot = createScatterPlot(containerElement, false, true);
+            this.scatterPlot = createScatterPlot(containerElement, window.ApplePaySession, true);
             if (this.props.chartOptions.dragmode === 'pan') {
                 this.scatterPlot.setInteractionMode('PAN');
             } else if (this.props.chartOptions.dragmode === 'select') {
@@ -378,7 +401,10 @@ class ScatterChartThree extends React.PureComponent {
                         }
 
                         if (typeof value === 'number') {
-                            value = numberFormat(value);
+                            value = numberFormat2f(value);
+                            if (value.endsWith('.00')) {
+                                value = value.substring(0, value.lastIndexOf('.'));
+                            }
                         }
                         this.tooltipElementRef.current.innerHTML = '' + value;
                     } else {
@@ -395,7 +421,7 @@ class ScatterChartThree extends React.PureComponent {
                 const widthHalf = this.props.chartSize.width / 2;
                 const heightHalf = this.props.chartSize.height / 2;
                 const pos = new Vector3();
-                const selectedPoints = [];
+                const selectedIndices = [];
 
                 for (let i = 0, j = 0, k = 0; i < traceInfo.npoints; i++, j += 4, k += 3) {
                     pos.x = positions[k];
@@ -405,17 +431,17 @@ class ScatterChartThree extends React.PureComponent {
                     pos.x = (pos.x * widthHalf) + widthHalf;
                     pos.y = -(pos.y * heightHalf) + heightHalf;
                     if (isPointInside(pos, points)) {
-                        selectedPoints.push(i);
+                        selectedIndices.push(i);
                     }
                 }
 
-                if (selectedPoints.length === 0) {
+                if (selectedIndices.length === 0) {
                     this.props.onDeselect({name: getEmbeddingKey(traceInfo.embedding)});
                 } else {
                     this.props.onSelected({
                         name: getEmbeddingKey(traceInfo.embedding),
                         clear: !this.props.chartOptions.editSelection,
-                        value: {basis: traceInfo.embedding, points: selectedPoints}
+                        value: {basis: traceInfo.embedding, points: selectedIndices}
                     });
                 }
             };
@@ -453,19 +479,39 @@ class ScatterChartThree extends React.PureComponent {
                     });
                 }
             };
+            const canvas = this.containerElementRef.current.querySelector('canvas');
+            canvas.style.outline = '0px';
+            const webglcontextlost = (e) => {
+                console.log('lost webgl context');
+                e.preventDefault();
+                this.scatterPlot = null;
+            };
+            const webglcontextrestored = (e) => {
+                console.log('restored webgl context');
+                e.preventDefault();
+                this.setState({forceUpdate: !this.state.forceUpdate});
+            };
+            canvas.addEventListener('webglcontextlost', webglcontextlost);
+            canvas.addEventListener('webglcontextrestored', webglcontextrestored);
         }
-        ;
 
-
-        const canvas = this.containerElementRef.current.querySelector('canvas');
-        canvas.style.outline = '0px';
     }
 
 
     draw() {
-        const {traceInfo, markerOpacity, unselectedMarkerOpacity, selection, pointSize, chartOptions, categoricalNames} = this.props;
+        const {
+            obsCat,
+            cachedData,
+            traceInfo,
+            markerOpacity,
+            unselectedMarkerOpacity,
+            selection,
+            pointSize,
+            chartOptions,
+            categoricalNames
+        } = this.props;
         updateScatterChart(this.scatterPlot, traceInfo, selection, markerOpacity, unselectedMarkerOpacity, pointSize,
-            categoricalNames, chartOptions);
+            categoricalNames, chartOptions, obsCat, cachedData);
     }
 
 
@@ -474,22 +520,21 @@ class ScatterChartThree extends React.PureComponent {
             <div className={this.props.classes.root}>
                 <ChartToolbar
                     dragmode={this.props.chartOptions.dragmode}
-                    animating={this.props.chartOptions.animating}
                     editSelection={this.props.chartOptions.editSelection}
-                    showLabels={this.props.chartOptions.showLabels}
+                    onGallery={this.props.onGallery}
+                    animating={this.props.chartOptions.animating}
                     showFog={this.props.chartOptions.showFog}
                     onShowFog={this.onShowFog}
-                    onMoreOptions={this.onMoreOptions}
                     is3d={this.props.traceInfo && this.props.traceInfo.z != null}
                     toggleAnimation={this.onToggleAnimation}
                     onSaveImage={this.onSaveImage}
-                    onShowLabels={this.onShowLabels}
                     onDragMode={this.onDragMode}
                     onCopyImage={this.onCopyImage}
                     onEditSelection={this.onEditSelection}
                     onShowAxis={this.onShowAxis}
+                    onHome={this.resetCamera}
                     showAxis={this.props.chartOptions.showAxis}
-                    onGallery={this.onGallery}>
+                >
                 </ChartToolbar>
                 <Typography color="textPrimary" ref={this.tooltipElementRef} style={{
                     display: 'inline-block',
