@@ -1,4 +1,3 @@
-import {scaleSequential} from 'd3-scale';
 import {isPlainObject} from 'lodash';
 import {combineReducers} from 'redux';
 import {
@@ -18,38 +17,39 @@ import {
     SET_DATASET_FILTER,
     SET_DATASET_FILTERS,
     SET_DIALOG,
+    SET_DISTRIBUTION_DATA,
+    SET_DISTRIBUTION_PLOT_INTERPOLATOR,
+    SET_DISTRIBUTION_PLOT_OPTIONS,
     SET_DOMAIN,
-    SET_DOT_PLOT_DATA,
-    SET_DOT_PLOT_SORT_ORDER,
     SET_EMAIL,
     SET_EMBEDDING_DATA,
+    SET_EMBEDDING_LABELS,
     SET_FEATURE_SUMMARY,
     SET_GLOBAL_FEATURE_SUMMARY,
     SET_INTERPOLATOR,
+    SET_JOB_RESULT,
+    SET_JOB_RESULTS,
     SET_LOADING,
     SET_LOADING_APP,
     SET_MARKER_OPACITY,
-    SET_MARKER_OPACITY_UI,
     SET_MARKERS,
     SET_MESSAGE,
     SET_NUMBER_OF_BINS,
-    SET_NUMBER_OF_BINS_UI,
     SET_POINT_SIZE,
     SET_PRIMARY_CHART_SIZE,
     SET_PRIMARY_TRACE_KEY,
     SET_SAVED_DATASET_STATE,
     SET_SEARCH_TOKENS,
-    SET_SELECTED_DOT_PLOT_DATA,
+    SET_SELECTED_DISTRIBUTION_DATA,
     SET_SELECTED_EMBEDDING,
     SET_SELECTION,
     SET_SERVER_INFO,
     SET_TAB,
     SET_UNSELECTED_MARKER_OPACITY,
-    SET_UNSELECTED_MARKER_OPACITY_UI,
     SET_USER,
     UPDATE_DATASET,
 } from '../actions';
-import {getInterpolator, updateTraceColors} from '../util';
+import {createColorScale, getInterpolator, updateTraceColors} from '../util';
 
 export const DEFAULT_BIN_SUMMARY = 'max';
 export const DEFAULT_NUMBER_BINS = 500;
@@ -57,6 +57,7 @@ export const DEFAULT_POINT_SIZE = 1;
 export const DEFAULT_MARKER_OPACITY = 1;
 export const DEFAULT_UNSELECTED_MARKER_OPACITY = 0.1;
 export const DEFAULT_INTERPOLATOR = 'Viridis';
+export const DEFAULT_DOT_PLOT_INTERPOLATOR = 'Reds';
 export const DEFAULT_DRAG_MODE = 'pan';
 export const DEFAULT_SHOW_LABELS = false;
 export const DEFAULT_SHOW_AXIS = true;
@@ -65,21 +66,36 @@ export const DEFAULT_DARK_MODE = false;
 export const DEFAULT_LABEL_FONT_SIZE = 14;
 export const DEFAULT_LABEL_STROKE_WIDTH = 4;
 
+const DEFAULT_DIST_PLOT_OPTIONS = {
+    chartType: 'dotplot',
+    violinScale: 'width',
+    violinHeight: 100,
+    violinWidth: 80,
+    violinShowBoxplot: true
+};
+
 const DEFAULT_INTERPOLATOR_OBJ = {
     name: DEFAULT_INTERPOLATOR,
-    value: getInterpolator(DEFAULT_INTERPOLATOR)
+    value: getInterpolator(DEFAULT_INTERPOLATOR),
+    reversed: false
 };
 
+const DEFAULT_DOT_PLOT_INTERPOLATOR_OBJ = {
+    name: DEFAULT_DOT_PLOT_INTERPOLATOR,
+    value: getInterpolator(DEFAULT_DOT_PLOT_INTERPOLATOR),
+    reversed: false
+};
+
+
 const DEFAULT_PRIMARY_CHART_SIZE = {
     width: window.innerWidth - 280,
-    height: Math.max(300, window.innerHeight - 220)
+    height: Math.max(300, window.innerHeight - 410)
 };
 
 const DEFAULT_CHART_OPTIONS = {
     animating: false,
     dragmode: DEFAULT_DRAG_MODE,
     editSelection: false,
-    showLabels: DEFAULT_SHOW_LABELS,
     showAxis: DEFAULT_SHOW_AXIS,
     showFog: DEFAULT_SHOW_FOG,
     darkMode: DEFAULT_DARK_MODE,
@@ -87,7 +103,7 @@ const DEFAULT_CHART_OPTIONS = {
     labelStrokeWidth: DEFAULT_LABEL_STROKE_WIDTH
 };
 
-function chartSize(state = 500, action) {
+function chartSize(state = 300, action) {
     switch (action.type) {
         case SET_CHART_SIZE:
             return action.payload;
@@ -125,6 +141,18 @@ function searchTokens(state = [], action) {
     }
 }
 
+function embeddingLabels(state = [], action) {
+    switch (action.type) {
+        case SET_EMBEDDING_LABELS:
+            return action.payload;
+        case SET_DATASET:
+            return [];
+        case RESTORE_VIEW:
+            return action.payload.embeddingLabels || [];
+        default:
+            return state;
+    }
+}
 
 function chartOptions(state = DEFAULT_CHART_OPTIONS, action) {
     switch (action.type) {
@@ -241,7 +269,7 @@ function markers(state = [], action) {
                 result = newResults;
             }
 
-            if (action.payload.markers_read_only != null) {
+            if (action.payload != null && action.payload.markers_read_only != null) {
                 if (isPlainObject(action.payload.markers_read_only)) { // old style, name => features
                     let markers = action.payload.markers_read_only;
                     for (let categoryName in markers) {
@@ -317,20 +345,6 @@ function numberOfBins(state = DEFAULT_NUMBER_BINS, action) {
     }
 }
 
-function numberOfBinsUI(state = DEFAULT_NUMBER_BINS, action) {
-    switch (action.type) {
-        case SET_NUMBER_OF_BINS:
-        case SET_NUMBER_OF_BINS_UI:
-            return action.payload;
-        case SET_DATASET:
-            return DEFAULT_NUMBER_BINS;
-        case RESTORE_VIEW:
-            return action.payload.numberOfBins != null ? action.payload.numberOfBins : state;
-        default:
-            return state;
-    }
-}
-
 
 function markerOpacity(state = DEFAULT_MARKER_OPACITY, action) {
     switch (action.type) {
@@ -340,7 +354,6 @@ function markerOpacity(state = DEFAULT_MARKER_OPACITY, action) {
             return DEFAULT_MARKER_OPACITY;
         case RESTORE_VIEW:
             return action.payload.markerOpacity != null ? action.payload.markerOpacity : state;
-
         default:
             return state;
     }
@@ -359,34 +372,6 @@ function unselectedMarkerOpacity(state = DEFAULT_UNSELECTED_MARKER_OPACITY, acti
     }
 }
 
-function markerOpacityUI(state = 1, action) {
-    switch (action.type) {
-        case SET_MARKER_OPACITY:
-        case SET_MARKER_OPACITY_UI:
-            return action.payload;
-        case SET_DATASET:
-            return DEFAULT_MARKER_OPACITY;
-        case RESTORE_VIEW:
-            return action.payload.markerOpacity != null ? action.payload.markerOpacity : state;
-        default:
-            return state;
-    }
-}
-
-function unselectedMarkerOpacityUI(state = DEFAULT_UNSELECTED_MARKER_OPACITY, action) {
-    switch (action.type) {
-        case SET_UNSELECTED_MARKER_OPACITY:
-        case SET_UNSELECTED_MARKER_OPACITY_UI:
-            return action.payload;
-        case SET_DATASET:
-            return DEFAULT_UNSELECTED_MARKER_OPACITY;
-        case RESTORE_VIEW:
-            return action.payload.unselectedMarkerOpacity != null ? action.payload.unselectedMarkerOpacity : state;
-        default:
-            return state;
-    }
-}
-
 
 function message(state = null, action) {
     switch (action.type) {
@@ -500,58 +485,63 @@ function dialog(state = null, action) {
     }
 }
 
-// [{
-//     "categories": array,
-//     "name": str,
-//     "fractionRange":[number],
-//     "meanRange":[number]
-//     "values": [{
-//         "name": str,
-//         "fractionExpressed": array
-//         "mean": array,
-//         "active": bool
-//     }]
-// }]
 
-function updateDotPlotDataRange(data) {
-    data.forEach(categoryItem => {
-        let fractionRange = [Number.MAX_VALUE, -Number.MAX_VALUE];
-        let meanRange = [Number.MAX_VALUE, -Number.MAX_VALUE];
-        categoryItem.values.forEach(feature => {
-            for (let i = 0, n = feature.mean.length; i < n; i++) {
-                fractionRange[0] = Math.min(feature.fractionExpressed[i], fractionRange[0]);
-                fractionRange[1] = Math.max(feature.fractionExpressed[i], fractionRange[1]);
-                meanRange[0] = Math.min(feature.mean[i], meanRange[0]);
-                meanRange[1] = Math.max(feature.mean[i], meanRange[1]);
-            }
-        });
-        categoryItem.meanRange = meanRange;
-        categoryItem.fractionRange = fractionRange;
-    });
+function dotPlotInterpolator(state = DEFAULT_DOT_PLOT_INTERPOLATOR_OBJ, action) {
+    switch (action.type) {
+        case SET_DISTRIBUTION_PLOT_INTERPOLATOR:
+            return action.payload;
+        case RESTORE_VIEW:
+            return action.payload.dotPlotInterpolator != null ? action.payload.dotPlotInterpolator : state;
+        default:
+            return state;
+    }
 }
 
-function dotPlotData(state = [], action) {
+
+/*
+sortBy, minSize, maxSize, min, max, chartType
+ */
+function distributionPlotOptions(state = DEFAULT_DIST_PLOT_OPTIONS, action) {
+    switch (action.type) {
+        case SET_DATASET:
+            return DEFAULT_DIST_PLOT_OPTIONS;
+        case SET_DISTRIBUTION_PLOT_OPTIONS:
+            return Object.assign({}, state, action.payload);
+        case RESTORE_VIEW:
+            return action.payload.distributionPlotOptions != null ? action.payload.distributionPlotOptions : state;
+        default:
+            return state;
+    }
+}
+
+function jobResult(state = null, action) {
+    switch (action.type) {
+        case SET_DATASET:
+            return null;
+        case SET_JOB_RESULT:
+            return action.payload;
+        default:
+            return state;
+    }
+}
+
+
+function jobResults(state = [], action) {
+    switch (action.type) {
+        case SET_DATASET:
+            return [];
+        case SET_JOB_RESULTS:
+            return action.payload;
+        default:
+            return state;
+    }
+}
+
+function distributionData(state = [], action) {
     switch (action.type) {
         case SET_CATEGORICAL_NAME:
-            for (let i = 0; i < state.length; i++) {
-                let item = state[i];
-                state[i] = Object.assign({}, item);
-            }
             return state.slice();
-        case SET_DOT_PLOT_SORT_ORDER:
-            const name = action.payload.name;
-            const sortBy = action.payload.value;
-            for (let i = 0; i < state.length; i++) {
-                let item = state[i];
-                if (item.name === name) {
-                    item.sortBy = sortBy;
-                    state[i] = Object.assign({}, item);
-                    break;
-                }
-            }
-            return state.slice();
-        case SET_DOT_PLOT_DATA:
-            updateDotPlotDataRange(action.payload);
+        case SET_DISTRIBUTION_DATA:
             return action.payload;
         case SET_DATASET:
             return [];
@@ -560,28 +550,11 @@ function dotPlotData(state = [], action) {
     }
 }
 
-function selectedDotPlotData(state = [], action) {
+function selectedDistributionData(state = [], action) {
     switch (action.type) {
         case SET_CATEGORICAL_NAME:
-            for (let i = 0; i < state.length; i++) {
-                let item = state[i];
-                state[i] = Object.assign({}, item);
-            }
             return state.slice();
-        case SET_DOT_PLOT_SORT_ORDER:
-            const name = action.payload.name;
-            const sortBy = action.payload.value;
-            for (let i = 0; i < state.length; i++) {
-                let item = state[i];
-                if (item.name === name) {
-                    item.sortBy = sortBy;
-                    state[i] = Object.assign({}, item);
-                    break;
-                }
-            }
-            return state.slice();
-        case SET_SELECTED_DOT_PLOT_DATA:
-            updateDotPlotDataRange(action.payload);
+        case SET_SELECTED_DISTRIBUTION_DATA:
             return action.payload;
         case SET_DATASET:
             return [];
@@ -652,6 +625,15 @@ function categoricalNames(state = {}, action) {
     }
 }
 
+// maps keys to values, features=>array, embedding key=>x,y,z, feature_embedding_key=>array for binned embeddings
+function cachedData(state = {}, action) {
+    switch (action.type) {
+        case SET_DATASET :
+            return {};
+    }
+    return state;
+
+}
 
 // each item has  data (list of traces, each trace has x, y, etc.), layout
 function embeddingData(state = [], action) {
@@ -694,7 +676,7 @@ function embeddingData(state = [], action) {
             state.forEach((traceInfo, stateIndex) => {
                 if (traceInfo.continuous) {
                     let domain = traceInfo.colorScale.domain();
-                    traceInfo.colorScale = scaleSequential(action.payload.value).domain(domain);
+                    traceInfo.colorScale = createColorScale(action.payload).domain(domain);
                     updateTraceColors(traceInfo);
                 }
             });
@@ -710,6 +692,8 @@ export function pointSize(state = DEFAULT_POINT_SIZE, action) {
     switch (action.type) {
         case SET_POINT_SIZE:
             return action.payload;
+        case RESTORE_VIEW:
+            return action.payload.pointSize != null ? action.payload.pointSize : state;
         default:
             return state;
     }
@@ -737,8 +721,7 @@ export function primaryTraceKey(state = null, action) {
             if (traces.length === 0) {
                 return null;
             }
-            const activeTrace = traces[traces.length - 1];
-            return getTraceKey(activeTrace); // last feature becomes primary
+            return getTraceKey(traces[traces.length - 1]); // last feature becomes primary
         default:
             return state;
     }
@@ -755,7 +738,6 @@ function loading(state = false, action) {
 
 
 function interpolator(state = DEFAULT_INTERPOLATOR_OBJ, action) {
-
     switch (action.type) {
         case SET_INTERPOLATOR:
             return action.payload;
@@ -769,40 +751,44 @@ function interpolator(state = DEFAULT_INTERPOLATOR_OBJ, action) {
 export default combineReducers({
     binSummary,
     binValues,
+    cachedData,
     categoricalNames,
     chartOptions,
     chartSize,
     combineDatasetFilters,
-    datasetFilter,
-    datasetFilters,
     dataset,
     datasetChoices,
+    datasetFilter,
+    datasetFilters,
     dialog,
-    dotPlotData,
+    distributionData,
+    distributionPlotOptions,
+    dotPlotInterpolator,
     email,
-    markers,
     embeddingData,
+    embeddingLabels,
     embeddings,
     featureSummary,
     globalFeatureSummary,
     interpolator,
+    jobResult,
+    jobResults,
     loading,
     loadingApp,
     markerOpacity,
-    markerOpacityUI,
-    numberOfBins,
-    numberOfBinsUI,
-    primaryChartSize,
+    markers,
     message,
+    numberOfBins,
     pointSize,
+    primaryChartSize,
     primaryTraceKey,
     savedDatasetState,
     searchTokens,
+    selectedDistributionData,
     selection,
-    selectedDotPlotData,
     serverInfo,
     tab,
     unselectedMarkerOpacity,
-    unselectedMarkerOpacityUI,
     user
+
 });
