@@ -1,7 +1,7 @@
 import pandas as pd
 from pytest import approx
 
-from cirrocumulus.data_processing import handle_stats
+from cirrocumulus.data_processing import handle_data
 
 
 def diff_measures(test_data, summary, fields, is_x):
@@ -36,29 +36,29 @@ def diff_dimensions(test_data, summary, fields):
 
 
 def test_measure(dataset_api, input_dataset, test_data, measures):
-    process_results = handle_stats(dataset_api=dataset_api, dataset=input_dataset, measures=measures)
+    process_results = handle_data(dataset_api=dataset_api, dataset=input_dataset, stats=dict(measures=measures))
     summary = process_results['summary']
     diff_measures(test_data, summary, measures, True)
 
 
 def test_continuous_obs(dataset_api, input_dataset, test_data, continuous_obs):
-    process_results = handle_stats(dataset_api=dataset_api, dataset=input_dataset,
-        measures=list(map(lambda x: 'obs/' + x, continuous_obs)))
+    process_results = handle_data(dataset_api=dataset_api, dataset=input_dataset,
+        stats=dict(measures=list(map(lambda x: 'obs/' + x, continuous_obs))))
     summary = process_results['summary']
     diff_measures(test_data, summary, continuous_obs, False)
 
 
 def test_dimension(dataset_api, input_dataset, test_data, dimensions):
-    process_results = handle_stats(dataset_api=dataset_api, dataset=input_dataset,
-        dimensions=dimensions, )
+    process_results = handle_data(dataset_api=dataset_api, dataset=input_dataset,
+        stats=dict(dimensions=dimensions))
     summary = process_results['summary']
     diff_dimensions(test_data, summary, dimensions)
 
 
 def test_all(dataset_api, input_dataset, test_data, measures, dimensions, continuous_obs):
-    process_results = handle_stats(dataset_api=dataset_api, dataset=input_dataset,
-        measures=measures + list(map(lambda x: 'obs/' + x, continuous_obs)),
-        dimensions=dimensions, )
+    process_results = handle_data(dataset_api=dataset_api, dataset=input_dataset,
+        stats=dict(measures=measures + list(map(lambda x: 'obs/' + x, continuous_obs)),
+            dimensions=dimensions))
     summary = process_results['summary']
     diff_measures(test_data, summary, measures, True)
     diff_measures(test_data, summary, continuous_obs, False)
